@@ -53,8 +53,8 @@ pub fn cli() -> clap::Command {
         )
         .arg(opt("base-rev", "Git revision to lookup for a baseline"))
         .arg(opt("head-rev", "Git revision with changes"))
-        .arg(flag("frozen", "Require Cargo.lock and cache are up to date").global(true))
-        .arg(flag("locked", "Require Cargo.lock is up to date").global(true))
+        .arg(flag("frozen", "Require Cargo.lock and cache to be up-to-date").global(true))
+        .arg(flag("locked", "Require Cargo.lock to be up-to-date").global(true))
         .arg(flag("offline", "Run without accessing the network").global(true))
         .arg(multi_opt("config", "KEY=VALUE", "Override a configuration value").global(true))
         .arg(
@@ -67,15 +67,15 @@ pub fn cli() -> clap::Command {
         )
 }
 
-pub fn exec(args: &clap::ArgMatches, config: &mut cargo::util::Config) -> cargo::CliResult {
-    config_configure(config, args)?;
+pub fn exec(args: &clap::ArgMatches, gctx: &mut cargo::util::GlobalContext) -> cargo::CliResult {
+    global_context_configure(gctx, args)?;
 
-    bump_check(args, config)?;
+    bump_check(args, gctx)?;
 
     Ok(())
 }
 
-fn config_configure(config: &mut Config, args: &ArgMatches) -> CliResult {
+fn global_context_configure(gctx: &mut GlobalContext, args: &ArgMatches) -> CliResult {
     let verbose = args.verbose();
     // quiet is unusual because it is redefined in some subcommands in order
     // to provide custom help text.
@@ -92,7 +92,7 @@ fn config_configure(config: &mut Config, args: &ArgMatches) -> CliResult {
     if let Some(values) = args.get_many::<String>("config") {
         config_args.extend(values.cloned());
     }
-    config.configure(
+    gctx.configure(
         verbose,
         quiet,
         color,
@@ -109,26 +109,35 @@ fn config_configure(config: &mut Config, args: &ArgMatches) -> CliResult {
 /// Main entry of `xtask-bump-check`.
 ///
 /// Assumption: version number are incremental. We never have point release for old versions.
-fn bump_check(args: &clap::ArgMatches, config: &cargo::util::Config) -> CargoResult<()> {
-    let ws = args.workspace(config)?;
+fn bump_check(args: &clap::ArgMatches, gctx: &cargo::util::GlobalContext) -> CargoResult<()> {
+    let ws = args.workspace(gctx)?;
     let repo = git2::Repository::open(ws.root())?;
-    let base_commit = get_base_commit(config, args, &repo)?;
+    let base_commit = get_base_commit(gctx, args, &repo)?;
     let head_commit = get_head_commit(args, &repo)?;
     let referenced_commit = get_referenced_commit(&repo, &base_commit)?;
     let changed_members = changed(&ws, &repo, &base_commit, &head_commit)?;
-    let status = |msg: &str| config.shell().status(STATUS, msg);
+    let status = |msg: &str| gctx.shell().status(STATUS, msg);
 
-    // Don't check against beta and stable branches,
-    // as the publish of these crates are not tied with Rust release process.
-    // See `TO_PUBLISH` in publish.py.
-    let crates_not_check_against_channels = ["home"];
+    let crates_not_check_against_channels = [
+        // High false positive rate between beta branch and requisite version bump soon after
+        //
+        // Low risk because we always bump the "major" version after beta branch; we are
+        // only losing out on checks for patch releases.
+        //
+        // Note: this is already skipped in `changed`
+        "cargo",
+        // Don't check against beta and stable branches,
+        // as the publish of these crates are not tied with Rust release process.
+        // See `TO_PUBLISH` in publish.py.
+        "home",
+    ];
 
     status(&format!("base commit `{}`", base_commit.id()))?;
     status(&format!("head commit `{}`", head_commit.id()))?;
 
     let mut needs_bump = Vec::new();
 
-    check_crates_io(config, &changed_members, &mut needs_bump)?;
+    check_crates_io(gctx, &changed_members, &mut needs_bump)?;
 
     if let Some(referenced_commit) = referenced_commit.as_ref() {
         status(&format!("compare against `{}`", referenced_commit.id()))?;
@@ -168,21 +177,24 @@ fn bump_check(args: &clap::ArgMatches, config: &cargo::util::Config) -> CargoRes
     let mut cmd = ProcessBuilder::new("cargo");
     cmd.arg("semver-checks")
         .arg("check-release")
+        .args(&["--exclude", "cargo-test-macro"]) // FIXME: Remove once 1.79 is stable.
+        .args(&["--exclude", "cargo-test-support"]) // FIXME: Remove once 1.79 is stable.
         .arg("--workspace");
-    config.shell().status("Running", &cmd)?;
+    gctx.shell().status("Running", &cmd)?;
     cmd.exec()?;
 
     if let Some(referenced_commit) = referenced_commit.as_ref() {
         let mut cmd = ProcessBuilder::new("cargo");
         cmd.arg("semver-checks")
             .arg("--workspace")
-            .args(&["--exclude", "rustfix"]) // FIXME: Remove once 1.76 is stable
+            .args(&["--exclude", "cargo-test-macro"]) // FIXME: Remove once 1.79 is stable.
+            .args(&["--exclude", "cargo-test-support"]) // FIXME: Remove once 1.79 is stable.
             .arg("--baseline-rev")
             .arg(referenced_commit.id().to_string());
         for krate in crates_not_check_against_channels {
             cmd.args(&["--exclude", krate]);
         }
-        config.shell().status("Running", &cmd)?;
+        gctx.shell().status("Running", &cmd)?;
         cmd.exec()?;
     }
 
@@ -193,7 +205,7 @@ fn bump_check(args: &clap::ArgMatches, config: &cargo::util::Config) -> CargoRes
 
 /// Returns the commit of upstream `master` branch if `base-rev` is missing.
 fn get_base_commit<'a>(
-    config: &Config,
+    gctx: &GlobalContext,
     args: &clap::ArgMatches,
     repo: &'a git2::Repository,
 ) -> CargoResult<git2::Commit<'a>> {
@@ -223,7 +235,7 @@ fn get_base_commit<'a>(
             let upstream_ref = upstream_branches[0].get();
             if upstream_branches.len() > 1 {
                 let name = upstream_ref.name().expect("name is valid UTF-8");
-                let _ = config.shell().warn(format!(
+                let _ = gctx.shell().warn(format!(
                     "multiple `{UPSTREAM_BRANCH}` found, picking {name}"
                 ));
             }
@@ -334,10 +346,8 @@ fn changed<'r, 'ws>(
             (relative_pkg_root, pkg)
         })
         .collect::<Vec<_>>();
-    let base_tree = base_commit.as_object().peel_to_tree()?;
-    let head_tree = head.as_object().peel_to_tree()?;
-    let diff = repo.diff_tree_to_tree(Some(&base_tree), Some(&head_tree), Default::default())?;
 
+    let diff = symmetric_diff(repo, base_commit, head)?;
     let mut changed_members = HashMap::new();
 
     for delta in diff.deltas() {
@@ -355,19 +365,35 @@ fn changed<'r, 'ws>(
     Ok(changed_members)
 }
 
+/// Using a "symmetric difference" between base and head.
+fn symmetric_diff<'a>(
+    repo: &'a git2::Repository,
+    base: &'a git2::Commit<'a>,
+    head: &'a git2::Commit<'a>,
+) -> CargoResult<git2::Diff<'a>> {
+    let ancestor_oid = repo.merge_base(base.id(), head.id())?;
+    let ancestor_commit = repo.find_commit(ancestor_oid)?;
+    let ancestor_tree = ancestor_commit.as_object().peel_to_tree()?;
+    let head_tree = head.as_object().peel_to_tree()?;
+    let diff =
+        repo.diff_tree_to_tree(Some(&ancestor_tree), Some(&head_tree), Default::default())?;
+    tracing::info!(merge_base = %ancestor_commit.id(), base = %base.id(), head = %head.id(), "git diff base...head");
+    Ok(diff)
+}
+
 /// Compares version against published crates on crates.io.
 ///
 /// Assumption: We always release a version larger than all existing versions.
 fn check_crates_io<'a>(
-    config: &Config,
+    gctx: &GlobalContext,
     changed_members: &HashMap<&'a str, &'a Package>,
     needs_bump: &mut Vec<&'a Package>,
 ) -> CargoResult<()> {
-    let source_id = SourceId::crates_io(config)?;
-    let mut registry = PackageRegistry::new(config)?;
-    let _lock = config.acquire_package_cache_lock(CacheLockMode::DownloadExclusive)?;
+    let source_id = SourceId::crates_io(gctx)?;
+    let mut registry = PackageRegistry::new(gctx)?;
+    let _lock = gctx.acquire_package_cache_lock(CacheLockMode::DownloadExclusive)?;
     registry.lock_patches();
-    config.shell().status(
+    gctx.shell().status(
         STATUS,
         format_args!("compare against `{}`", source_id.display_registry_name()),
     )?;
@@ -403,11 +429,11 @@ fn check_crates_io<'a>(
 }
 
 /// Checkouts a temporary workspace to do further version comparisons.
-fn checkout_ws<'cfg, 'a>(
-    ws: &Workspace<'cfg>,
+fn checkout_ws<'gctx, 'a>(
+    ws: &Workspace<'gctx>,
     repo: &'a git2::Repository,
     referenced_commit: &git2::Commit<'a>,
-) -> CargoResult<Workspace<'cfg>> {
+) -> CargoResult<Workspace<'gctx>> {
     let repo_path = repo.path().as_os_str().to_str().unwrap();
     // Put it under `target/cargo-<short-id>`
     let short_id = &referenced_commit.id().to_string()[..7];
@@ -419,7 +445,7 @@ fn checkout_ws<'cfg, 'a>(
         .clone(repo_path, checkout_path)?;
     let obj = new_repo.find_object(referenced_commit.id(), None)?;
     new_repo.reset(&obj, git2::ResetType::Hard, None)?;
-    Workspace::new(&checkout_path.join("Cargo.toml"), ws.config())
+    Workspace::new(&checkout_path.join("Cargo.toml"), ws.gctx())
 }
 
 #[test]
