@@ -9,12 +9,12 @@ Print a JSON representation of a Cargo.toml manifest.
 Deprecated, use `<cyan,bold>cargo metadata --no-deps</>` instead.\
 "
         ))
-        .arg_quiet()
+        .arg_silent_suggestion()
         .arg_manifest_path()
 }
 
-pub fn exec(config: &mut Config, args: &ArgMatches) -> CliResult {
-    let ws = args.workspace(config)?;
-    config.shell().print_json(&ws.current()?.serialized())?;
+pub fn exec(gctx: &mut GlobalContext, args: &ArgMatches) -> CliResult {
+    let ws = args.workspace(gctx)?;
+    gctx.shell().print_json(&ws.current()?.serialized())?;
     Ok(())
 }
