@@ -10,7 +10,7 @@
 //!   that would compile other Cargo targets. It will recursively creates units
 //!   of its dependencies. One biggest difference is that the [`Unit`] of
 //!   compiling a build script is flagged as [`TargetKind::CustomBuild`].
-//! * Build script executaion --- During the construction of the [`UnitGraph`],
+//! * Build script execution --- During the construction of the [`UnitGraph`],
 //!   Cargo inserts a [`Unit`] with [`CompileMode::RunCustomBuild`]. This unit
 //!   depends on the unit of compiling the associated build script, to ensure
 //!   the executable is available before running. The [`Work`] of running the
@@ -22,7 +22,7 @@
 //! of a build script. Standard output is the chosen interprocess communication
 //! between Cargo and build script processes. A set of strings is defined for
 //! that purpose. These strings, a.k.a. instructions, are interpreted by
-//! [`BuildOutput::parse`] and stored in [`Context::build_script_outputs`].
+//! [`BuildOutput::parse`] and stored in [`BuildRunner::build_script_outputs`].
 //! The entire execution work is constructed by [`build_work`].
 //!
 //! [build script]: https://doc.rust-lang.org/nightly/cargo/reference/build-scripts.html
@@ -31,29 +31,35 @@
 //! [`CompileMode::RunCustomBuild`]: super::CompileMode
 //! [instructions]: https://doc.rust-lang.org/cargo/reference/build-scripts.html#outputs-of-the-build-script
 
-use super::{fingerprint, Context, Job, Unit, Work};
+use super::{fingerprint, BuildRunner, Job, Unit, Work};
 use crate::core::compiler::artifact;
-use crate::core::compiler::context::Metadata;
+use crate::core::compiler::build_runner::Metadata;
+use crate::core::compiler::fingerprint::DirtyReason;
 use crate::core::compiler::job_queue::JobState;
 use crate::core::{profiles::ProfileRoot, PackageId, Target};
 use crate::util::errors::CargoResult;
+use crate::util::internal;
 use crate::util::machine_message::{self, Message};
-use crate::util::{internal, profile};
 use anyhow::{bail, Context as _};
 use cargo_platform::Cfg;
 use cargo_util::paths;
+use cargo_util_schemas::manifest::RustVersion;
 use std::collections::hash_map::{Entry, HashMap};
 use std::collections::{BTreeSet, HashSet};
 use std::path::{Path, PathBuf};
-use std::str;
+use std::str::{self, FromStr};
 use std::sync::{Arc, Mutex};
 
+/// Deprecated: A build script instruction that tells Cargo to display a warning after the
+/// build script has finished running. Read [the doc] for more.
+///
+/// [the doc]: https://doc.rust-lang.org/nightly/cargo/reference/build-scripts.html#cargo-warning
+const OLD_CARGO_WARNING_SYNTAX: &str = "cargo:warning=";
 /// A build script instruction that tells Cargo to display a warning after the
 /// build script has finished running. Read [the doc] for more.
 ///
 /// [the doc]: https://doc.rust-lang.org/nightly/cargo/reference/build-scripts.html#cargo-warning
-const CARGO_WARNING: &str = "cargo:warning=";
-
+const NEW_CARGO_WARNING_SYNTAX: &str = "cargo::warning=";
 /// Contains the parsed output of a custom build script.
 #[derive(Clone, Debug, Hash, Default)]
 pub struct BuildOutput {
@@ -146,7 +152,7 @@ pub struct BuildDeps {
     pub rerun_if_env_changed: Vec<String>,
 }
 
-/// Represents one of the instructions from `cargo:rustc-link-arg-*` build
+/// Represents one of the instructions from `cargo::rustc-link-arg-*` build
 /// script instruction family.
 ///
 /// In other words, indicates targets that custom linker arguments applies to.
@@ -156,19 +162,19 @@ pub struct BuildDeps {
 /// [1]: https://doc.rust-lang.org/nightly/cargo/reference/build-scripts.html#cargorustc-link-argflag
 #[derive(Clone, Hash, Debug, PartialEq, Eq)]
 pub enum LinkArgTarget {
-    /// Represents `cargo:rustc-link-arg=FLAG`.
+    /// Represents `cargo::rustc-link-arg=FLAG`.
     All,
-    /// Represents `cargo:rustc-cdylib-link-arg=FLAG`.
+    /// Represents `cargo::rustc-cdylib-link-arg=FLAG`.
     Cdylib,
-    /// Represents `cargo:rustc-link-arg-bins=FLAG`.
+    /// Represents `cargo::rustc-link-arg-bins=FLAG`.
     Bin,
-    /// Represents `cargo:rustc-link-arg-bin=BIN=FLAG`.
+    /// Represents `cargo::rustc-link-arg-bin=BIN=FLAG`.
     SingleBin(String),
-    /// Represents `cargo:rustc-link-arg-tests=FLAG`.
+    /// Represents `cargo::rustc-link-arg-tests=FLAG`.
     Test,
-    /// Represents `cargo:rustc-link-arg-benches=FLAG`.
+    /// Represents `cargo::rustc-link-arg-benches=FLAG`.
     Bench,
-    /// Represents `cargo:rustc-link-arg-examples=FLAG`.
+    /// Represents `cargo::rustc-link-arg-examples=FLAG`.
     Example,
 }
 
@@ -188,24 +194,19 @@ impl LinkArgTarget {
 }
 
 /// Prepares a `Work` that executes the target as a custom build script.
-pub fn prepare(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
-    let _p = profile::start(format!(
-        "build script prepare: {}/{}",
-        unit.pkg,
-        unit.target.name()
-    ));
-
-    let metadata = cx.get_run_build_script_metadata(unit);
-    if cx
+#[tracing::instrument(skip_all)]
+pub fn prepare(build_runner: &mut BuildRunner<'_, '_>, unit: &Unit) -> CargoResult<Job> {
+    let metadata = build_runner.get_run_build_script_metadata(unit);
+    if build_runner
         .build_script_outputs
         .lock()
         .unwrap()
         .contains_key(metadata)
     {
         // The output is already set, thus the build script is overridden.
-        fingerprint::prepare_target(cx, unit, false)
+        fingerprint::prepare_target(build_runner, unit, false)
     } else {
-        build_work(cx, unit)
+        build_work(build_runner, unit)
     }
 }
 
@@ -224,7 +225,7 @@ fn emit_build_output(
         .collect::<Vec<_>>();
 
     let msg = machine_message::BuildScript {
-        package_id,
+        package_id: package_id.to_spec(),
         linked_libs: &output.library_links,
         linked_paths: &library_paths,
         cfgs: &output.cfgs,
@@ -244,23 +245,23 @@ fn emit_build_output(
 /// * Create the output dir (`OUT_DIR`) for the build script output.
 /// * Determine if the build script needs a re-run.
 /// * Run the build script and store its output.
-fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
+fn build_work(build_runner: &mut BuildRunner<'_, '_>, unit: &Unit) -> CargoResult<Job> {
     assert!(unit.mode.is_run_custom_build());
-    let bcx = &cx.bcx;
-    let dependencies = cx.unit_deps(unit);
+    let bcx = &build_runner.bcx;
+    let dependencies = build_runner.unit_deps(unit);
     let build_script_unit = dependencies
         .iter()
         .find(|d| !d.unit.mode.is_run_custom_build() && d.unit.target.is_custom_build())
         .map(|d| &d.unit)
         .expect("running a script not depending on an actual script");
-    let script_dir = cx.files().build_script_dir(build_script_unit);
-    let script_out_dir = cx.files().build_script_out_dir(unit);
-    let script_run_dir = cx.files().build_script_run_dir(unit);
+    let script_dir = build_runner.files().build_script_dir(build_script_unit);
+    let script_out_dir = build_runner.files().build_script_out_dir(unit);
+    let script_run_dir = build_runner.files().build_script_run_dir(unit);
     let build_plan = bcx.build_config.build_plan;
     let invocation_name = unit.buildkey();
 
     if let Some(deps) = unit.pkg.manifest().metabuild() {
-        prepare_metabuild(cx, build_script_unit, deps)?;
+        prepare_metabuild(build_runner, build_script_unit, deps)?;
     }
 
     // Building the command to execute
@@ -274,7 +275,7 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
     // `Profiles::get_profile_run_custom_build` so that those flags get
     // carried over.
     let to_exec = to_exec.into_os_string();
-    let mut cmd = cx.compilation.host_process(to_exec, &unit.pkg)?;
+    let mut cmd = build_runner.compilation.host_process(to_exec, &unit.pkg)?;
     let debug = unit.profile.debuginfo.is_turned_on();
     cmd.env("OUT_DIR", &script_out_dir)
         .env("CARGO_MANIFEST_DIR", unit.pkg.root())
@@ -291,15 +292,15 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
         )
         .env("HOST", &bcx.host_triple())
         .env("RUSTC", &bcx.rustc().path)
-        .env("RUSTDOC", &*bcx.config.rustdoc()?)
-        .inherit_jobserver(&cx.jobserver);
+        .env("RUSTDOC", &*bcx.gctx.rustdoc()?)
+        .inherit_jobserver(&build_runner.jobserver);
 
     // Find all artifact dependencies and make their file and containing directory discoverable using environment variables.
-    for (var, value) in artifact::get_env(cx, dependencies)? {
+    for (var, value) in artifact::get_env(build_runner, dependencies)? {
         cmd.env(&var, value);
     }
 
-    if let Some(linker) = &cx.compilation.target_linker(unit.kind) {
+    if let Some(linker) = &build_runner.compilation.target_linker(unit.kind) {
         cmd.env("RUSTC_LINKER", linker);
     }
 
@@ -346,7 +347,7 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
         cmd.env_remove("RUSTC_WRAPPER");
     }
     cmd.env_remove("RUSTC_WORKSPACE_WRAPPER");
-    if cx.bcx.ws.is_member(&unit.pkg) {
+    if build_runner.bcx.ws.is_member(&unit.pkg) {
         if let Some(wrapper) = bcx.rustc().workspace_wrapper.as_ref() {
             cmd.env("RUSTC_WORKSPACE_WRAPPER", wrapper);
         }
@@ -357,7 +358,7 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
     );
     cmd.env_remove("RUSTFLAGS");
 
-    if cx.bcx.ws.config().extra_verbose() {
+    if build_runner.bcx.ws.gctx().extra_verbose() {
         cmd.display_env_vars();
     }
 
@@ -370,7 +371,7 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
         .iter()
         .filter_map(|dep| {
             if dep.unit.mode.is_run_custom_build() {
-                let dep_metadata = cx.get_run_build_script_metadata(&dep.unit);
+                let dep_metadata = build_runner.get_run_build_script_metadata(&dep.unit);
                 Some((
                     dep.unit.pkg.manifest().links().unwrap().to_string(),
                     dep.unit.pkg.package_id(),
@@ -383,12 +384,12 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
         .collect::<Vec<_>>();
     let library_name = unit.pkg.library().map(|t| t.crate_name());
     let pkg_descr = unit.pkg.to_string();
-    let build_script_outputs = Arc::clone(&cx.build_script_outputs);
+    let build_script_outputs = Arc::clone(&build_runner.build_script_outputs);
     let id = unit.pkg.package_id();
     let output_file = script_run_dir.join("output");
     let err_file = script_run_dir.join("stderr");
     let root_output_file = script_run_dir.join("root-output");
-    let host_target_root = cx.files().host_dest().to_path_buf();
+    let host_target_root = build_runner.files().host_dest().to_path_buf();
     let all = (
         id,
         library_name.clone(),
@@ -397,23 +398,29 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
         output_file.clone(),
         script_out_dir.clone(),
     );
-    let build_scripts = cx.build_scripts.get(unit).cloned();
+    let build_scripts = build_runner.build_scripts.get(unit).cloned();
     let json_messages = bcx.build_config.emit_json();
-    let extra_verbose = bcx.config.extra_verbose();
-    let (prev_output, prev_script_out_dir) = prev_build_output(cx, unit);
-    let metadata_hash = cx.get_run_build_script_metadata(unit);
+    let extra_verbose = bcx.gctx.extra_verbose();
+    let (prev_output, prev_script_out_dir) = prev_build_output(build_runner, unit);
+    let metadata_hash = build_runner.get_run_build_script_metadata(unit);
 
     paths::create_dir_all(&script_dir)?;
     paths::create_dir_all(&script_out_dir)?;
 
-    let nightly_features_allowed = cx.bcx.config.nightly_features_allowed;
-    let extra_check_cfg = cx.bcx.config.cli_unstable().check_cfg;
+    let nightly_features_allowed = build_runner.bcx.gctx.nightly_features_allowed;
+    let extra_check_cfg = build_runner
+        .bcx
+        .target_data
+        .info(unit.kind)
+        .support_check_cfg;
     let targets: Vec<Target> = unit.pkg.targets().to_vec();
+    let msrv = unit.pkg.rust_version().cloned();
     // Need a separate copy for the fresh closure.
     let targets_fresh = targets.clone();
+    let msrv_fresh = msrv.clone();
 
     let env_profile_name = unit.profile.name.to_uppercase();
-    let built_with_debuginfo = cx
+    let built_with_debuginfo = build_runner
         .bcx
         .unit_graph
         .get(unit)
@@ -478,7 +485,10 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
         let output = cmd
             .exec_with_streaming(
                 &mut |stdout| {
-                    if let Some(warning) = stdout.strip_prefix(CARGO_WARNING) {
+                    if let Some(warning) = stdout
+                        .strip_prefix(OLD_CARGO_WARNING_SYNTAX)
+                        .or(stdout.strip_prefix(NEW_CARGO_WARNING_SYNTAX))
+                    {
                         warnings_in_case_of_panic.push(warning.to_owned());
                     }
                     if extra_verbose {
@@ -553,6 +563,7 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
             extra_check_cfg,
             nightly_features_allowed,
             &targets,
+            &msrv,
         )?;
 
         if json_messages {
@@ -581,6 +592,7 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
                 extra_check_cfg,
                 nightly_features_allowed,
                 &targets_fresh,
+                &msrv_fresh,
             )?,
         };
 
@@ -595,10 +607,10 @@ fn build_work(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Job> {
         Ok(())
     });
 
-    let mut job = if cx.bcx.build_config.build_plan {
-        Job::new_dirty(Work::noop(), None)
+    let mut job = if build_runner.bcx.build_config.build_plan {
+        Job::new_dirty(Work::noop(), DirtyReason::FreshBuild)
     } else {
-        fingerprint::prepare_target(cx, unit, false)?
+        fingerprint::prepare_target(build_runner, unit, false)?
     };
     if job.freshness().is_dirty() {
         job.before(dirty);
@@ -637,6 +649,7 @@ impl BuildOutput {
         extra_check_cfg: bool,
         nightly_features_allowed: bool,
         targets: &[Target],
+        msrv: &Option<RustVersion>,
     ) -> CargoResult<BuildOutput> {
         let contents = paths::read_bytes(path)?;
         BuildOutput::parse(
@@ -648,6 +661,7 @@ impl BuildOutput {
             extra_check_cfg,
             nightly_features_allowed,
             targets,
+            msrv,
         )
     }
 
@@ -655,9 +669,7 @@ impl BuildOutput {
     ///
     /// * `pkg_descr` --- for error messages
     /// * `library_name` --- for determining if `RUSTC_BOOTSTRAP` should be allowed
-    /// * `extra_check_cfg` --- for unstable feature [`-Zcheck-cfg`]
-    ///
-    /// [`-Zcheck-cfg`]: https://doc.rust-lang.org/cargo/reference/unstable.html#check-cfg
+    /// * `extra_check_cfg` --- for `--check-cfg` (if supported)
     pub fn parse(
         input: &[u8],
         // Takes String instead of InternedString so passing `unit.pkg.name()` will give a compile error.
@@ -668,6 +680,7 @@ impl BuildOutput {
         extra_check_cfg: bool,
         nightly_features_allowed: bool,
         targets: &[Target],
+        msrv: &Option<RustVersion>,
     ) -> CargoResult<BuildOutput> {
         let mut library_paths = Vec::new();
         let mut library_links = Vec::new();
@@ -680,50 +693,155 @@ impl BuildOutput {
         let mut rerun_if_env_changed = Vec::new();
         let mut warnings = Vec::new();
         let whence = format!("build script of `{}`", pkg_descr);
+        // Old syntax:
+        //    cargo:rustc-flags=VALUE
+        //    cargo:KEY=VALUE (for other unreserved keys)
+        // New syntax:
+        //    cargo::rustc-flags=VALUE
+        //    cargo::metadata=KEY=VALUE (for other unreserved keys)
+        // Due to backwards compatibility, no new keys can be added to this old format.
+        const RESERVED_PREFIXES: &[&str] = &[
+            "rustc-flags=",
+            "rustc-link-lib=",
+            "rustc-link-search=",
+            "rustc-link-arg-cdylib=",
+            "rustc-cdylib-link-arg=",
+            "rustc-link-arg-bins=",
+            "rustc-link-arg-bin=",
+            "rustc-link-arg-tests=",
+            "rustc-link-arg-benches=",
+            "rustc-link-arg-examples=",
+            "rustc-link-arg=",
+            "rustc-cfg=",
+            "rustc-check-cfg=",
+            "rustc-env=",
+            "warning=",
+            "rerun-if-changed=",
+            "rerun-if-env-changed=",
+        ];
+        const DOCS_LINK_SUGGESTION: &str = "See https://doc.rust-lang.org/cargo/reference/build-scripts.html#outputs-of-the-build-script \
+                for more information about build script outputs.";
+
+        fn has_reserved_prefix(flag: &str) -> bool {
+            RESERVED_PREFIXES
+                .iter()
+                .any(|reserved_prefix| flag.starts_with(reserved_prefix))
+        }
+
+        fn check_minimum_supported_rust_version_for_new_syntax(
+            pkg_descr: &str,
+            msrv: &Option<RustVersion>,
+            flag: &str,
+        ) -> CargoResult<()> {
+            if let Some(msrv) = msrv {
+                let new_syntax_added_in = RustVersion::from_str("1.77.0")?;
+                if !new_syntax_added_in.is_compatible_with(msrv.as_partial()) {
+                    let old_syntax_suggestion = if has_reserved_prefix(flag) {
+                        format!(
+                            "Switch to the old `cargo:{flag}` syntax (note the single colon).\n"
+                        )
+                    } else if flag.starts_with("metadata=") {
+                        let old_format_flag = flag.strip_prefix("metadata=").unwrap();
+                        format!("Switch to the old `cargo:{old_format_flag}` syntax instead of `cargo::{flag}` (note the single colon).\n")
+                    } else {
+                        String::new()
+                    };
+
+                    bail!(
+                        "the `cargo::` syntax for build script output instructions was added in \
+                        Rust 1.77.0, but the minimum supported Rust version of `{pkg_descr}` is {msrv}.\n\
+                        {old_syntax_suggestion}\
+                        {DOCS_LINK_SUGGESTION}"
+                    );
+                }
+            }
+
+            Ok(())
+        }
+
+        fn parse_directive<'a>(
+            whence: &str,
+            line: &str,
+            data: &'a str,
+            old_syntax: bool,
+        ) -> CargoResult<(&'a str, &'a str)> {
+            let mut iter = data.splitn(2, "=");
+            let key = iter.next();
+            let value = iter.next();
+            match (key, value) {
+                (Some(a), Some(b)) => Ok((a, b.trim_end())),
+                _ => bail!(
+                    "invalid output in {whence}: `{line}`\n\
+                    Expected a line with `{syntax}KEY=VALUE` with an `=` character, \
+                    but none was found.\n\
+                    {DOCS_LINK_SUGGESTION}",
+                    syntax = if old_syntax { "cargo:" } else { "cargo::" },
+                ),
+            }
+        }
+
+        fn parse_metadata<'a>(
+            whence: &str,
+            line: &str,
+            data: &'a str,
+            old_syntax: bool,
+        ) -> CargoResult<(&'a str, &'a str)> {
+            let mut iter = data.splitn(2, "=");
+            let key = iter.next();
+            let value = iter.next();
+            match (key, value) {
+                (Some(a), Some(b)) => Ok((a, b.trim_end())),
+                _ => bail!(
+                    "invalid output in {whence}: `{line}`\n\
+                    Expected a line with `{syntax}KEY=VALUE` with an `=` character, \
+                    but none was found.\n\
+                    {DOCS_LINK_SUGGESTION}",
+                    syntax = if old_syntax {
+                        "cargo:"
+                    } else {
+                        "cargo::metadata="
+                    },
+                ),
+            }
+        }
 
         for line in input.split(|b| *b == b'\n') {
             let line = match str::from_utf8(line) {
                 Ok(line) => line.trim(),
                 Err(..) => continue,
             };
-            let data = match line.split_once(':') {
-                Some(("cargo", val)) => {
-                    if val.starts_with(":") {
-                        // Line started with `cargo::`.
-                        bail!("unsupported output in {whence}: `{line}`\n\
-                            Found a `cargo::key=value` build directive which is reserved for future use.\n\
-                            Either change the directive to `cargo:key=value` syntax (note the single `:`) or upgrade your version of Rust.\n\
-                            See https://doc.rust-lang.org/cargo/reference/build-scripts.html#outputs-of-the-build-script \
-                            for more information about build script outputs.");
-                    }
-                    val
+            let mut old_syntax = false;
+            let (key, value) = if let Some(data) = line.strip_prefix("cargo::") {
+                check_minimum_supported_rust_version_for_new_syntax(pkg_descr, msrv, data)?;
+                // For instance, `cargo::rustc-flags=foo` or `cargo::metadata=foo=bar`.
+                parse_directive(whence.as_str(), line, data, old_syntax)?
+            } else if let Some(data) = line.strip_prefix("cargo:") {
+                old_syntax = true;
+                // For instance, `cargo:rustc-flags=foo`.
+                if has_reserved_prefix(data) {
+                    parse_directive(whence.as_str(), line, data, old_syntax)?
+                } else {
+                    // For instance, `cargo:foo=bar`.
+                    ("metadata", data)
                 }
-                _ => continue,
+            } else {
+                // Skip this line since it doesn't start with "cargo:" or "cargo::".
+                continue;
             };
-
-            // getting the `key=value` part of the line
-            let (key, value) = match data.split_once('=') {
-                Some((a,b)) => (a, b.trim_end()),
-                // Line started with `cargo:` but didn't match `key=value`.
-                _ => bail!("invalid output in {}: `{}`\n\
-                    Expected a line with `cargo:key=value` with an `=` character, \
-                    but none was found.\n\
-                    See https://doc.rust-lang.org/cargo/reference/build-scripts.html#outputs-of-the-build-script \
-                    for more information about build script outputs.", whence, line),
-            };
-
             // This will rewrite paths if the target directory has been moved.
             let value = value.replace(
                 script_out_dir_when_generated.to_str().unwrap(),
                 script_out_dir.to_str().unwrap(),
             );
 
+            let syntax_prefix = if old_syntax { "cargo:" } else { "cargo::" };
             macro_rules! check_and_add_target {
                 ($target_kind: expr, $is_target_kind: expr, $link_type: expr) => {
                     if !targets.iter().any(|target| $is_target_kind(target)) {
                         bail!(
-                            "invalid instruction `cargo:{}` from {}\n\
+                            "invalid instruction `{}{}` from {}\n\
                                 The package {} does not have a {} target.",
+                            syntax_prefix,
                             key,
                             whence,
                             pkg_descr,
@@ -746,14 +864,14 @@ impl BuildOutput {
                 "rustc-link-arg-cdylib" | "rustc-cdylib-link-arg" => {
                     if !targets.iter().any(|target| target.is_cdylib()) {
                         warnings.push(format!(
-                            "cargo:{} was specified in the build script of {}, \
+                            "{}{} was specified in the build script of {}, \
                              but that package does not contain a cdylib target\n\
                              \n\
                              Allowing this was an unintended change in the 1.50 \
                              release, and may become an error in the future. \
                              For more information, see \
                              <https://github.com/rust-lang/cargo/issues/9562>.",
-                            key, pkg_descr
+                            syntax_prefix, key, pkg_descr
                         ));
                     }
                     linker_args.push((LinkArgTarget::Cdylib, value))
@@ -764,11 +882,13 @@ impl BuildOutput {
                 "rustc-link-arg-bin" => {
                     let (bin_name, arg) = value.split_once('=').ok_or_else(|| {
                         anyhow::format_err!(
-                            "invalid instruction `cargo:{}={}` from {}\n\
-                                The instruction should have the form cargo:{}=BIN=ARG",
+                            "invalid instruction `{}{}={}` from {}\n\
+                                The instruction should have the form {}{}=BIN=ARG",
+                            syntax_prefix,
                             key,
                             value,
                             whence,
+                            syntax_prefix,
                             key
                         )
                     })?;
@@ -777,8 +897,9 @@ impl BuildOutput {
                         .any(|target| target.is_bin() && target.name() == bin_name)
                     {
                         bail!(
-                            "invalid instruction `cargo:{}` from {}\n\
+                            "invalid instruction `{}{}` from {}\n\
                                 The package {} does not have a bin target with the name `{}`.",
+                            syntax_prefix,
                             key,
                             whence,
                             pkg_descr,
@@ -807,7 +928,8 @@ impl BuildOutput {
                     if extra_check_cfg {
                         check_cfgs.push(value.to_string());
                     } else {
-                        warnings.push(format!("cargo:{} requires -Zcheck-cfg flag", key));
+                        // silently ignoring the instruction because the rustc version
+                        // we are using does not support --check-cfg stably
                     }
                 }
                 "rustc-env" => {
@@ -864,7 +986,15 @@ impl BuildOutput {
                 "warning" => warnings.push(value.to_string()),
                 "rerun-if-changed" => rerun_if_changed.push(PathBuf::from(value)),
                 "rerun-if-env-changed" => rerun_if_env_changed.push(value.to_string()),
-                _ => metadata.push((key.to_string(), value.to_string())),
+                "metadata" => {
+                    let (key, value) = parse_metadata(whence.as_str(), line, &value, old_syntax)?;
+                    metadata.push((key.to_owned(), value.to_owned()));
+                }
+                _ => bail!(
+                    "invalid output in {whence}: `{line}`\n\
+                    Unknown key: `{key}`.\n\
+                    {DOCS_LINK_SUGGESTION}",
+                ),
             }
         }
 
@@ -882,9 +1012,9 @@ impl BuildOutput {
         })
     }
 
-    /// Parses [`cargo:rustc-flags`] instruction.
+    /// Parses [`cargo::rustc-flags`] instruction.
     ///
-    /// [`cargo:rustc-flags`]: https://doc.rust-lang.org/nightly/cargo/reference/build-scripts.html#cargorustc-flagsflags
+    /// [`cargo::rustc-flags`]: https://doc.rust-lang.org/nightly/cargo/reference/build-scripts.html#cargorustc-flagsflags
     pub fn parse_rustc_flags(
         value: &str,
         whence: &str,
@@ -930,9 +1060,9 @@ impl BuildOutput {
         Ok((library_paths, library_links))
     }
 
-    /// Parses [`cargo:rustc-env`] instruction.
+    /// Parses [`cargo::rustc-env`] instruction.
     ///
-    /// [`cargo:rustc-env`]: https://doc.rust-lang.org/nightly/cargo/reference/build-scripts.html#rustc-env
+    /// [`cargo::rustc-env`]: https://doc.rust-lang.org/nightly/cargo/reference/build-scripts.html#rustc-env
     pub fn parse_rustc_env(value: &str, whence: &str) -> CargoResult<(String, String)> {
         match value.split_once('=') {
             Some((n, v)) => Ok((n.to_owned(), v.to_owned())),
@@ -944,9 +1074,13 @@ impl BuildOutput {
 /// Prepares the Rust script for the unstable feature [metabuild].
 ///
 /// [metabuild]: https://doc.rust-lang.org/nightly/cargo/reference/unstable.html#metabuild
-fn prepare_metabuild(cx: &Context<'_, '_>, unit: &Unit, deps: &[String]) -> CargoResult<()> {
+fn prepare_metabuild(
+    build_runner: &BuildRunner<'_, '_>,
+    unit: &Unit,
+    deps: &[String],
+) -> CargoResult<()> {
     let mut output = Vec::new();
-    let available_deps = cx.unit_deps(unit);
+    let available_deps = build_runner.unit_deps(unit);
     // Filter out optional dependencies, and look up the actual lib name.
     let meta_deps: Vec<_> = deps
         .iter()
@@ -957,16 +1091,16 @@ fn prepare_metabuild(cx: &Context<'_, '_>, unit: &Unit, deps: &[String]) -> Carg
                 .map(|d| d.unit.target.crate_name())
         })
         .collect();
-    for dep in &meta_deps {
-        output.push(format!("use {};\n", dep));
-    }
     output.push("fn main() {\n".to_string());
     for dep in &meta_deps {
         output.push(format!("    {}::metabuild();\n", dep));
     }
     output.push("}\n".to_string());
     let output = output.join("");
-    let path = unit.pkg.manifest().metabuild_path(cx.bcx.ws.target_dir());
+    let path = unit
+        .pkg
+        .manifest()
+        .metabuild_path(build_runner.bcx.ws.target_dir());
     paths::create_dir_all(path.parent().unwrap())?;
     paths::write_if_changed(path, &output)?;
     Ok(())
@@ -990,7 +1124,7 @@ impl BuildDeps {
     }
 }
 
-/// Computes several maps in [`Context`].
+/// Computes several maps in [`BuildRunner`].
 ///
 /// - [`build_scripts`]: A map that tracks which build scripts each package
 ///   depends on.
@@ -1008,15 +1142,16 @@ impl BuildDeps {
 /// The given set of units to this function is the initial set of
 /// targets/profiles which are being built.
 ///
-/// [`build_scripts`]: Context::build_scripts
-/// [`build_explicit_deps`]: Context::build_explicit_deps
-/// [`build_script_outputs`]: Context::build_script_outputs
-pub fn build_map(cx: &mut Context<'_, '_>) -> CargoResult<()> {
+/// [`build_scripts`]: BuildRunner::build_scripts
+/// [`build_explicit_deps`]: BuildRunner::build_explicit_deps
+/// [`build_script_outputs`]: BuildRunner::build_script_outputs
+pub fn build_map(build_runner: &mut BuildRunner<'_, '_>) -> CargoResult<()> {
     let mut ret = HashMap::new();
-    for unit in &cx.bcx.roots {
-        build(&mut ret, cx, unit)?;
+    for unit in &build_runner.bcx.roots {
+        build(&mut ret, build_runner, unit)?;
     }
-    cx.build_scripts
+    build_runner
+        .build_scripts
         .extend(ret.into_iter().map(|(k, v)| (k, Arc::new(v))));
     return Ok(());
 
@@ -1024,7 +1159,7 @@ pub fn build_map(cx: &mut Context<'_, '_>) -> CargoResult<()> {
     // memoizes all of its return values as it goes along.
     fn build<'a>(
         out: &'a mut HashMap<Unit, BuildScripts>,
-        cx: &mut Context<'_, '_>,
+        build_runner: &mut BuildRunner<'_, '_>,
         unit: &Unit,
     ) -> CargoResult<&'a BuildScripts> {
         // Do a quick pre-flight check to see if we've already calculated the
@@ -1036,9 +1171,13 @@ pub fn build_map(cx: &mut Context<'_, '_>) -> CargoResult<()> {
         // If there is a build script override, pre-fill the build output.
         if unit.mode.is_run_custom_build() {
             if let Some(links) = unit.pkg.manifest().links() {
-                if let Some(output) = cx.bcx.target_data.script_override(links, unit.kind) {
-                    let metadata = cx.get_run_build_script_metadata(unit);
-                    cx.build_script_outputs.lock().unwrap().insert(
+                if let Some(output) = build_runner
+                    .bcx
+                    .target_data
+                    .script_override(links, unit.kind)
+                {
+                    let metadata = build_runner.get_run_build_script_metadata(unit);
+                    build_runner.build_script_outputs.lock().unwrap().insert(
                         unit.pkg.package_id(),
                         metadata,
                         output.clone(),
@@ -1051,26 +1190,29 @@ pub fn build_map(cx: &mut Context<'_, '_>) -> CargoResult<()> {
 
         // If a package has a build script, add itself as something to inspect for linking.
         if !unit.target.is_custom_build() && unit.pkg.has_custom_build() {
-            let script_meta = cx
+            let script_meta = build_runner
                 .find_build_script_metadata(unit)
                 .expect("has_custom_build should have RunCustomBuild");
             add_to_link(&mut ret, unit.pkg.package_id(), script_meta);
         }
 
         if unit.mode.is_run_custom_build() {
-            parse_previous_explicit_deps(cx, unit);
+            parse_previous_explicit_deps(build_runner, unit);
         }
 
         // We want to invoke the compiler deterministically to be cache-friendly
         // to rustc invocation caching schemes, so be sure to generate the same
         // set of build script dependency orderings via sorting the targets that
         // come out of the `Context`.
-        let mut dependencies: Vec<Unit> =
-            cx.unit_deps(unit).iter().map(|d| d.unit.clone()).collect();
+        let mut dependencies: Vec<Unit> = build_runner
+            .unit_deps(unit)
+            .iter()
+            .map(|d| d.unit.clone())
+            .collect();
         dependencies.sort_by_key(|u| u.pkg.package_id());
 
         for dep_unit in dependencies.iter() {
-            let dep_scripts = build(out, cx, dep_unit)?;
+            let dep_scripts = build(out, build_runner, dep_unit)?;
 
             if dep_unit.target.for_host() {
                 ret.plugins.extend(dep_scripts.to_link.iter().cloned());
@@ -1096,12 +1238,12 @@ pub fn build_map(cx: &mut Context<'_, '_>) -> CargoResult<()> {
     }
 
     /// Load any dependency declarations from a previous build script run.
-    fn parse_previous_explicit_deps(cx: &mut Context<'_, '_>, unit: &Unit) {
-        let script_run_dir = cx.files().build_script_run_dir(unit);
+    fn parse_previous_explicit_deps(build_runner: &mut BuildRunner<'_, '_>, unit: &Unit) {
+        let script_run_dir = build_runner.files().build_script_run_dir(unit);
         let output_file = script_run_dir.join("output");
-        let (prev_output, _) = prev_build_output(cx, unit);
+        let (prev_output, _) = prev_build_output(build_runner, unit);
         let deps = BuildDeps::new(&output_file, prev_output.as_ref());
-        cx.build_explicit_deps.insert(unit.clone(), deps);
+        build_runner.build_explicit_deps.insert(unit.clone(), deps);
     }
 }
 
@@ -1110,9 +1252,12 @@ pub fn build_map(cx: &mut Context<'_, '_>) -> CargoResult<()> {
 ///
 /// Also returns the directory containing the output, typically used later in
 /// processing.
-fn prev_build_output(cx: &mut Context<'_, '_>, unit: &Unit) -> (Option<BuildOutput>, PathBuf) {
-    let script_out_dir = cx.files().build_script_out_dir(unit);
-    let script_run_dir = cx.files().build_script_run_dir(unit);
+fn prev_build_output(
+    build_runner: &mut BuildRunner<'_, '_>,
+    unit: &Unit,
+) -> (Option<BuildOutput>, PathBuf) {
+    let script_out_dir = build_runner.files().build_script_out_dir(unit);
+    let script_run_dir = build_runner.files().build_script_run_dir(unit);
     let root_output_file = script_run_dir.join("root-output");
     let output_file = script_run_dir.join("output");
 
@@ -1127,9 +1272,14 @@ fn prev_build_output(cx: &mut Context<'_, '_>, unit: &Unit) -> (Option<BuildOutp
             &unit.pkg.to_string(),
             &prev_script_out_dir,
             &script_out_dir,
-            cx.bcx.config.cli_unstable().check_cfg,
-            cx.bcx.config.nightly_features_allowed,
+            build_runner
+                .bcx
+                .target_data
+                .info(unit.kind)
+                .support_check_cfg,
+            build_runner.bcx.gctx.nightly_features_allowed,
             unit.pkg.targets(),
+            &unit.pkg.rust_version().cloned(),
         )
         .ok(),
         prev_script_out_dir,
