@@ -24,12 +24,19 @@ use crate::sources::{PathSource, CRATES_IO_INDEX, CRATES_IO_REGISTRY};
 use crate::util::edit_distance;
 use crate::util::errors::{CargoResult, ManifestError};
 use crate::util::interning::InternedString;
+use crate::util::lints::{
+    analyze_cargo_lints_table, check_im_a_teapot, check_implicit_features, unused_dependencies,
+};
 use crate::util::toml::{read_manifest, InheritableFields};
-use crate::util::{config::ConfigRelativePath, Config, Filesystem, IntoUrl};
-use crate::util_schemas::manifest::RustVersion;
-use crate::util_schemas::manifest::{TomlDependency, TomlProfiles};
+use crate::util::{
+    context::CargoResolverConfig, context::CargoResolverPrecedence, context::ConfigRelativePath,
+    Filesystem, GlobalContext, IntoUrl,
+};
 use cargo_util::paths;
 use cargo_util::paths::normalize_path;
+use cargo_util_schemas::manifest;
+use cargo_util_schemas::manifest::RustVersion;
+use cargo_util_schemas::manifest::{TomlDependency, TomlProfiles};
 use pathdiff::diff_paths;
 
 /// The core abstraction in Cargo for working with a workspace of crates.
@@ -38,8 +45,8 @@ use pathdiff::diff_paths;
 /// other functions. It's typically through this object that the current
 /// package is loaded and/or learned about.
 #[derive(Debug)]
-pub struct Workspace<'cfg> {
-    config: &'cfg Config,
+pub struct Workspace<'gctx> {
+    gctx: &'gctx GlobalContext,
 
     // This path is a path to where the current cargo subcommand was invoked
     // from. That is the `--manifest-path` argument to Cargo, and
@@ -48,7 +55,7 @@ pub struct Workspace<'cfg> {
 
     // A list of packages found in this workspace. Always includes at least the
     // package mentioned by `current_manifest`.
-    packages: Packages<'cfg>,
+    packages: Packages<'gctx>,
 
     // If this workspace includes more than one crate, this points to the root
     // of the workspace. This is `None` in the case that `[workspace]` is
@@ -98,6 +105,7 @@ pub struct Workspace<'cfg> {
 
     /// The resolver behavior specified with the `resolver` field.
     resolve_behavior: ResolveBehavior,
+    resolve_honors_rust_version: bool,
 
     /// Workspace-level custom metadata
     custom_metadata: Option<toml::Value>,
@@ -106,8 +114,8 @@ pub struct Workspace<'cfg> {
 // Separate structure for tracking loaded packages (to avoid loading anything
 // twice), and this is separate to help appease the borrow checker.
 #[derive(Debug)]
-struct Packages<'cfg> {
-    config: &'cfg Config,
+struct Packages<'gctx> {
+    gctx: &'gctx GlobalContext,
     packages: HashMap<PathBuf, MaybePackage>,
 }
 
@@ -180,16 +188,16 @@ pub struct WorkspaceRootConfig {
     custom_metadata: Option<toml::Value>,
 }
 
-impl<'cfg> Workspace<'cfg> {
+impl<'gctx> Workspace<'gctx> {
     /// Creates a new workspace given the target manifest pointed to by
     /// `manifest_path`.
     ///
     /// This function will construct the entire workspace by determining the
     /// root and all member packages. It will then validate the workspace
     /// before returning it, so `Ok` is only returned for valid workspaces.
-    pub fn new(manifest_path: &Path, config: &'cfg Config) -> CargoResult<Workspace<'cfg>> {
-        let mut ws = Workspace::new_default(manifest_path.to_path_buf(), config);
-        ws.target_dir = config.target_dir()?;
+    pub fn new(manifest_path: &Path, gctx: &'gctx GlobalContext) -> CargoResult<Workspace<'gctx>> {
+        let mut ws = Workspace::new_default(manifest_path.to_path_buf(), gctx);
+        ws.target_dir = gctx.target_dir()?;
 
         if manifest_path.is_relative() {
             bail!(
@@ -204,17 +212,17 @@ impl<'cfg> Workspace<'cfg> {
             .load_workspace_config()?
             .and_then(|cfg| cfg.custom_metadata);
         ws.find_members()?;
-        ws.set_resolve_behavior();
+        ws.set_resolve_behavior()?;
         ws.validate()?;
         Ok(ws)
     }
 
-    fn new_default(current_manifest: PathBuf, config: &'cfg Config) -> Workspace<'cfg> {
+    fn new_default(current_manifest: PathBuf, gctx: &'gctx GlobalContext) -> Workspace<'gctx> {
         Workspace {
-            config,
+            gctx,
             current_manifest,
             packages: Packages {
-                config,
+                gctx,
                 packages: HashMap::new(),
             },
             root_manifest: None,
@@ -227,6 +235,7 @@ impl<'cfg> Workspace<'cfg> {
             loaded_packages: RefCell::new(HashMap::new()),
             ignore_lock: false,
             resolve_behavior: ResolveBehavior::V1,
+            resolve_honors_rust_version: false,
             custom_metadata: None,
         }
     }
@@ -235,16 +244,16 @@ impl<'cfg> Workspace<'cfg> {
         root_path: PathBuf,
         current_manifest: PathBuf,
         manifest: VirtualManifest,
-        config: &'cfg Config,
-    ) -> CargoResult<Workspace<'cfg>> {
-        let mut ws = Workspace::new_default(current_manifest, config);
+        gctx: &'gctx GlobalContext,
+    ) -> CargoResult<Workspace<'gctx>> {
+        let mut ws = Workspace::new_default(current_manifest, gctx);
         ws.root_manifest = Some(root_path.join("Cargo.toml"));
-        ws.target_dir = config.target_dir()?;
+        ws.target_dir = gctx.target_dir()?;
         ws.packages
             .packages
             .insert(root_path, MaybePackage::Virtual(manifest));
         ws.find_members()?;
-        ws.set_resolve_behavior();
+        ws.set_resolve_behavior()?;
         // TODO: validation does not work because it walks up the directory
         // tree looking for the root which is a fake file that doesn't exist.
         Ok(ws)
@@ -261,11 +270,11 @@ impl<'cfg> Workspace<'cfg> {
     /// `cargo package`.
     pub fn ephemeral(
         package: Package,
-        config: &'cfg Config,
+        gctx: &'gctx GlobalContext,
         target_dir: Option<Filesystem>,
         require_optional_deps: bool,
-    ) -> CargoResult<Workspace<'cfg>> {
-        let mut ws = Workspace::new_default(package.manifest_path().to_path_buf(), config);
+    ) -> CargoResult<Workspace<'gctx>> {
+        let mut ws = Workspace::new_default(package.manifest_path().to_path_buf(), gctx);
         ws.is_ephemeral = true;
         ws.require_optional_deps = require_optional_deps;
         let key = ws.current_manifest.parent().unwrap();
@@ -275,16 +284,16 @@ impl<'cfg> Workspace<'cfg> {
         ws.target_dir = if let Some(dir) = target_dir {
             Some(dir)
         } else {
-            ws.config.target_dir()?
+            ws.gctx.target_dir()?
         };
         ws.members.push(ws.current_manifest.clone());
         ws.member_ids.insert(id);
         ws.default_members.push(ws.current_manifest.clone());
-        ws.set_resolve_behavior();
+        ws.set_resolve_behavior()?;
         Ok(ws)
     }
 
-    fn set_resolve_behavior(&mut self) {
+    fn set_resolve_behavior(&mut self) -> CargoResult<()> {
         // - If resolver is specified in the workspace definition, use that.
         // - If the root package specifies the resolver, use that.
         // - If the root package specifies edition 2021, use v2.
@@ -295,7 +304,44 @@ impl<'cfg> Workspace<'cfg> {
                 .resolve_behavior()
                 .unwrap_or_else(|| p.manifest().edition().default_resolve_behavior()),
             MaybePackage::Virtual(vm) => vm.resolve_behavior().unwrap_or(ResolveBehavior::V1),
+        };
+
+        match self.resolve_behavior() {
+            ResolveBehavior::V1 | ResolveBehavior::V2 => {}
+            ResolveBehavior::V3 => {
+                if self.resolve_behavior == ResolveBehavior::V3 {
+                    self.resolve_honors_rust_version = true;
+                }
+            }
         }
+        match self.gctx().get::<CargoResolverConfig>("resolver") {
+            Ok(CargoResolverConfig {
+                something_like_precedence: Some(precedence),
+            }) => {
+                if self.gctx().cli_unstable().msrv_policy {
+                    self.resolve_honors_rust_version =
+                        precedence == CargoResolverPrecedence::SomethingLikeRustVersion;
+                } else {
+                    self.gctx()
+                        .shell()
+                        .warn("ignoring `resolver` config table without `-Zmsrv-policy`")?;
+                }
+            }
+            Ok(CargoResolverConfig {
+                something_like_precedence: None,
+            }) => {}
+            Err(err) => {
+                if self.gctx().cli_unstable().msrv_policy {
+                    return Err(err);
+                } else {
+                    self.gctx()
+                        .shell()
+                        .warn("ignoring `resolver` config table without `-Zmsrv-policy`")?;
+                }
+            }
+        }
+
+        Ok(())
     }
 
     /// Returns the current package of this workspace.
@@ -349,9 +395,9 @@ impl<'cfg> Workspace<'cfg> {
         }
     }
 
-    /// Returns the `Config` this workspace is associated with.
-    pub fn config(&self) -> &'cfg Config {
-        self.config
+    /// Returns the `GlobalContext` this workspace is associated with.
+    pub fn gctx(&self) -> &'gctx GlobalContext {
+        self.gctx
     }
 
     pub fn profiles(&self) -> Option<&TomlProfiles> {
@@ -396,7 +442,7 @@ impl<'cfg> Workspace<'cfg> {
             rel_path.push(&hash[0..2]);
             rel_path.push(&hash[2..]);
 
-            self.config().home().join(rel_path)
+            self.gctx().home().join(rel_path)
         } else {
             Filesystem::new(self.root().join("target"))
         }
@@ -415,19 +461,18 @@ impl<'cfg> Workspace<'cfg> {
     fn config_patch(&self) -> CargoResult<HashMap<Url, Vec<Dependency>>> {
         let config_patch: Option<
             BTreeMap<String, BTreeMap<String, TomlDependency<ConfigRelativePath>>>,
-        > = self.config.get("patch")?;
+        > = self.gctx.get("patch")?;
 
         let source = SourceId::for_path(self.root())?;
 
         let mut warnings = Vec::new();
-        let mut nested_paths = Vec::new();
 
         let mut patch = HashMap::new();
         for (url, deps) in config_patch.into_iter().flatten() {
             let url = match &url[..] {
                 CRATES_IO_REGISTRY => CRATES_IO_INDEX.parse().unwrap(),
                 url => self
-                    .config
+                    .gctx
                     .get_registry_index(url)
                     .or_else(|_| url.into_url())
                     .with_context(|| {
@@ -442,14 +487,12 @@ impl<'cfg> Workspace<'cfg> {
                             dep,
                             name,
                             source,
-                            &mut nested_paths,
-                            self.config,
+                            self.gctx,
                             &mut warnings,
                             /* platform */ None,
                             // NOTE: Since we use ConfigRelativePath, this root isn't used as
                             // any relative paths are resolved before they'd be joined with root.
                             Path::new("unused-relative-path"),
-                            self.unstable_features(),
                             /* kind */ None,
                         )
                     })
@@ -458,7 +501,7 @@ impl<'cfg> Workspace<'cfg> {
         }
 
         for message in warnings {
-            self.config
+            self.gctx
                 .shell()
                 .warn(format!("[patch] in cargo config: {}", message))?
         }
@@ -586,7 +629,7 @@ impl<'cfg> Workspace<'cfg> {
     pub fn set_require_optional_deps(
         &mut self,
         require_optional_deps: bool,
-    ) -> &mut Workspace<'cfg> {
+    ) -> &mut Workspace<'gctx> {
         self.require_optional_deps = require_optional_deps;
         self
     }
@@ -595,7 +638,7 @@ impl<'cfg> Workspace<'cfg> {
         self.ignore_lock
     }
 
-    pub fn set_ignore_lock(&mut self, ignore_lock: bool) -> &mut Workspace<'cfg> {
+    pub fn set_ignore_lock(&mut self, ignore_lock: bool) -> &mut Workspace<'gctx> {
         self.ignore_lock = ignore_lock;
         self
     }
@@ -606,6 +649,16 @@ impl<'cfg> Workspace<'cfg> {
         self.members().filter_map(|pkg| pkg.rust_version()).min()
     }
 
+    pub fn set_resolve_honors_rust_version(&mut self, honor_rust_version: Option<bool>) {
+        if let Some(honor_rust_version) = honor_rust_version {
+            self.resolve_honors_rust_version = honor_rust_version;
+        }
+    }
+
+    pub fn resolve_honors_rust_version(&self) -> bool {
+        self.resolve_honors_rust_version
+    }
+
     pub fn custom_metadata(&self) -> Option<&toml::Value> {
         self.custom_metadata.as_ref()
     }
@@ -649,7 +702,7 @@ impl<'cfg> Workspace<'cfg> {
                 debug!("find_root - is root {}", manifest_path.display());
                 Ok(Some(root_path))
             }
-            None => find_workspace_root_with_loader(manifest_path, self.config, |self_path| {
+            None => find_workspace_root_with_loader(manifest_path, self.gctx, |self_path| {
                 Ok(self
                     .packages
                     .load(self_path)?
@@ -697,8 +750,10 @@ impl<'cfg> Workspace<'cfg> {
             self.find_path_deps(&path.join("Cargo.toml"), &root_manifest_path, false)
                 .with_context(|| {
                     format!(
-                        "failed to load manifest for workspace member `{}`",
-                        path.display()
+                        "failed to load manifest for workspace member `{}`\n\
+                        referenced by workspace at `{}`",
+                        path.display(),
+                        root_manifest_path.display()
                     )
                 })?;
         }
@@ -722,9 +777,10 @@ impl<'cfg> Workspace<'cfg> {
                         continue;
                     }
                     bail!(
-                        "package `{}` is listed in workspace’s default-members \
-                         but is not a member.",
-                        path.display()
+                        "package `{}` is listed in default-members but is not a member\n\
+                        for workspace at {}.",
+                        path.display(),
+                        root_manifest_path.display()
                     )
                 }
                 self.default_members.push(manifest_path)
@@ -769,7 +825,7 @@ impl<'cfg> Workspace<'cfg> {
             }
         }
 
-        debug!("find_members - {}", manifest_path.display());
+        debug!("find_path_deps - {}", manifest_path.display());
         self.members.push(manifest_path.clone());
 
         let candidates = {
@@ -817,7 +873,7 @@ impl<'cfg> Workspace<'cfg> {
         self.is_virtual()
             || match self.resolve_behavior() {
                 ResolveBehavior::V1 => false,
-                ResolveBehavior::V2 => true,
+                ResolveBehavior::V2 | ResolveBehavior::V3 => true,
             }
     }
 
@@ -1001,9 +1057,9 @@ impl<'cfg> Workspace<'cfg> {
                         pkg.manifest_path().display(),
                         root_manifest.display(),
                     );
-                    self.config.shell().warn(&msg)
+                    self.gctx.shell().warn(&msg)
                 };
-                if manifest.original().has_profiles() {
+                if manifest.resolved_toml().has_profiles() {
                     emit_warning("profiles")?;
                 }
                 if !manifest.replace().is_empty() {
@@ -1029,16 +1085,16 @@ impl<'cfg> Workspace<'cfg> {
                         .max()
                     {
                         let resolver = edition.default_resolve_behavior().to_manifest();
-                        self.config.shell().warn(format_args!(
+                        self.gctx.shell().warn(format_args!(
                             "virtual workspace defaulting to `resolver = \"1\"` despite one or more workspace members being on edition {edition} which implies `resolver = \"{resolver}\"`"
                         ))?;
-                        self.config.shell().note(
+                        self.gctx.shell().note(
                             "to keep the current resolver, specify `workspace.resolver = \"1\"` in the workspace root's manifest",
                         )?;
-                        self.config.shell().note(format_args!(
+                        self.gctx.shell().note(format_args!(
                             "to use the edition {edition} resolver, specify `workspace.resolver = \"{resolver}\"` in the workspace root's manifest"
                         ))?;
-                        self.config.shell().note(
+                        self.gctx.shell().note(
                             "for more details see https://doc.rust-lang.org/cargo/reference/resolver.html#resolver-versions",
                         )?;
                     }
@@ -1060,7 +1116,7 @@ impl<'cfg> Workspace<'cfg> {
             return Ok(p);
         }
         let source_id = SourceId::for_path(manifest_path.parent().unwrap())?;
-        let (package, _nested_paths) = ops::read_package(manifest_path, source_id, self.config)?;
+        let package = ops::read_package(manifest_path, source_id, self.gctx)?;
         loaded.insert(manifest_path.to_path_buf(), package.clone());
         Ok(package)
     }
@@ -1071,7 +1127,7 @@ impl<'cfg> Workspace<'cfg> {
     /// for various operations, and this preload step avoids doubly-loading and
     /// parsing crates on the filesystem by inserting them all into the registry
     /// with their in-memory formats.
-    pub fn preload(&self, registry: &mut PackageRegistry<'cfg>) {
+    pub fn preload(&self, registry: &mut PackageRegistry<'gctx>) {
         // These can get weird as this generally represents a workspace during
         // `cargo install`. Things like git repositories will actually have a
         // `PathSource` with multiple entries in it, so the logic below is
@@ -1086,19 +1142,23 @@ impl<'cfg> Workspace<'cfg> {
                 MaybePackage::Package(ref p) => p.clone(),
                 MaybePackage::Virtual(_) => continue,
             };
-            let mut src = PathSource::new(pkg.root(), pkg.package_id().source_id(), self.config);
-            src.preload_with(pkg);
+            let src = PathSource::preload_with(pkg, self.gctx);
             registry.add_preloaded(Box::new(src));
         }
     }
 
     pub fn emit_warnings(&self) -> CargoResult<()> {
         for (path, maybe_pkg) in &self.packages.packages {
+            let path = path.join("Cargo.toml");
+            if let MaybePackage::Package(pkg) = maybe_pkg {
+                if self.gctx.cli_unstable().cargo_lints {
+                    self.emit_lints(pkg, &path)?
+                }
+            }
             let warnings = match maybe_pkg {
                 MaybePackage::Package(pkg) => pkg.manifest().warnings().warnings(),
                 MaybePackage::Virtual(vm) => vm.warnings().warnings(),
             };
-            let path = path.join("Cargo.toml");
             for warning in warnings {
                 if warning.is_critical {
                     let err = anyhow::format_err!("{}", warning.message);
@@ -1113,13 +1173,59 @@ impl<'cfg> Workspace<'cfg> {
                         // originated, so include the path.
                         format!("{}: {}", path.display(), warning.message)
                     };
-                    self.config.shell().warn(msg)?
+                    self.gctx.shell().warn(msg)?
                 }
             }
         }
         Ok(())
     }
 
+    pub fn emit_lints(&self, pkg: &Package, path: &Path) -> CargoResult<()> {
+        let mut error_count = 0;
+        let toml_lints = pkg
+            .manifest()
+            .resolved_toml()
+            .lints
+            .clone()
+            .map(|lints| lints.lints)
+            .unwrap_or(manifest::TomlLints::default());
+        let cargo_lints = toml_lints
+            .get("cargo")
+            .cloned()
+            .unwrap_or(manifest::TomlToolLints::default());
+
+        let ws_contents = match self.root_maybe() {
+            MaybePackage::Package(pkg) => pkg.manifest().contents(),
+            MaybePackage::Virtual(v) => v.contents(),
+        };
+
+        let ws_document = match self.root_maybe() {
+            MaybePackage::Package(pkg) => pkg.manifest().document(),
+            MaybePackage::Virtual(v) => v.document(),
+        };
+
+        analyze_cargo_lints_table(
+            pkg,
+            &path,
+            &cargo_lints,
+            ws_contents,
+            ws_document,
+            self.root_manifest(),
+            self.gctx,
+        )?;
+        check_im_a_teapot(pkg, &path, &cargo_lints, &mut error_count, self.gctx)?;
+        check_implicit_features(pkg, &path, &cargo_lints, &mut error_count, self.gctx)?;
+        unused_dependencies(pkg, &path, &cargo_lints, &mut error_count, self.gctx)?;
+        if error_count > 0 {
+            Err(crate::util::errors::AlreadyPrintedError::new(anyhow!(
+                "encountered {error_count} errors(s) while running lints"
+            ))
+            .into())
+        } else {
+            Ok(())
+        }
+    }
+
     pub fn set_target_dir(&mut self, target_dir: Filesystem) {
         self.target_dir = Some(target_dir);
     }
@@ -1570,7 +1676,7 @@ impl<'cfg> Workspace<'cfg> {
     }
 }
 
-impl<'cfg> Packages<'cfg> {
+impl<'gctx> Packages<'gctx> {
     fn get(&self, manifest_path: &Path) -> &MaybePackage {
         self.maybe_get(manifest_path).unwrap()
     }
@@ -1593,8 +1699,7 @@ impl<'cfg> Packages<'cfg> {
             Entry::Occupied(e) => Ok(e.into_mut()),
             Entry::Vacant(v) => {
                 let source_id = SourceId::for_path(key)?;
-                let (manifest, _nested_paths) =
-                    read_manifest(manifest_path, source_id, self.config)?;
+                let manifest = read_manifest(manifest_path, source_id, self.gctx)?;
                 Ok(v.insert(match manifest {
                     EitherManifest::Real(manifest) => {
                         MaybePackage::Package(Package::new(manifest, manifest_path))
@@ -1737,11 +1842,14 @@ pub fn resolve_relative_path(
 }
 
 /// Finds the path of the root of the workspace.
-pub fn find_workspace_root(manifest_path: &Path, config: &Config) -> CargoResult<Option<PathBuf>> {
-    find_workspace_root_with_loader(manifest_path, config, |self_path| {
+pub fn find_workspace_root(
+    manifest_path: &Path,
+    gctx: &GlobalContext,
+) -> CargoResult<Option<PathBuf>> {
+    find_workspace_root_with_loader(manifest_path, gctx, |self_path| {
         let key = self_path.parent().unwrap();
         let source_id = SourceId::for_path(key)?;
-        let (manifest, _nested_paths) = read_manifest(self_path, source_id, config)?;
+        let manifest = read_manifest(self_path, source_id, gctx)?;
         Ok(manifest
             .workspace_config()
             .get_ws_root(self_path, manifest_path))
@@ -1754,12 +1862,12 @@ pub fn find_workspace_root(manifest_path: &Path, config: &Config) -> CargoResult
 /// workspace root is.
 fn find_workspace_root_with_loader(
     manifest_path: &Path,
-    config: &Config,
+    gctx: &GlobalContext,
     mut loader: impl FnMut(&Path) -> CargoResult<Option<PathBuf>>,
 ) -> CargoResult<Option<PathBuf>> {
     // Check if there are any workspace roots that have already been found that would work
     {
-        let roots = config.ws_roots.borrow();
+        let roots = gctx.ws_roots.borrow();
         // Iterate through the manifests parent directories until we find a workspace
         // root. Note we skip the first item since that is just the path itself
         for current in manifest_path.ancestors().skip(1) {
@@ -1772,7 +1880,7 @@ fn find_workspace_root_with_loader(
         }
     }
 
-    for ances_manifest_path in find_root_iter(manifest_path, config) {
+    for ances_manifest_path in find_root_iter(manifest_path, gctx) {
         debug!("find_root - trying {}", ances_manifest_path.display());
         if let Some(ws_root_path) = loader(&ances_manifest_path)? {
             return Ok(Some(ws_root_path));
@@ -1793,7 +1901,7 @@ fn read_root_pointer(member_manifest: &Path, root_link: &str) -> PathBuf {
 
 fn find_root_iter<'a>(
     manifest_path: &'a Path,
-    config: &'a Config,
+    gctx: &'a GlobalContext,
 ) -> impl Iterator<Item = PathBuf> + 'a {
     LookBehind::new(paths::ancestors(manifest_path, None).skip(2))
         .take_while(|path| !path.curr.ends_with("target/package"))
@@ -1804,7 +1912,7 @@ fn find_root_iter<'a>(
         // crates.io crates into the workspace by accident.
         .take_while(|path| {
             if let Some(last) = path.last {
-                config.home() != last
+                gctx.home() != last
             } else {
                 true
             }
