@@ -5,7 +5,7 @@
 //! rough outline is:
 //!
 //! 1. Resolve the dependency graph (see [`ops::resolve`]).
-//! 2. Download any packages needed (see [`PackageSet`].
+//! 2. Download any packages needed (see [`PackageSet`]).
 //! 3. Generate a list of top-level "units" of work for the targets the user
 //!   requested on the command-line. Each [`Unit`] corresponds to a compiler
 //!   invocation. This is done in this module ([`UnitGenerator::generate_root_units`]).
@@ -13,7 +13,7 @@
 //!   from the resolver.  See also [`unit_dependencies`].
 //! 5. Construct the [`BuildContext`] with all of the information collected so
 //!   far. This is the end of the "front end" of compilation.
-//! 6. Create a [`Context`] which coordinates the compilation process
+//! 6. Create a [`BuildRunner`] which coordinates the compilation process
 //!   and will perform the following steps:
 //!     1. Prepare the `target` directory (see [`Layout`]).
 //!     2. Create a [`JobQueue`]. The queue checks the
@@ -43,7 +43,7 @@ use std::sync::Arc;
 use crate::core::compiler::unit_dependencies::build_unit_dependencies;
 use crate::core::compiler::unit_graph::{self, UnitDep, UnitGraph};
 use crate::core::compiler::{standard_lib, CrateType, TargetInfo};
-use crate::core::compiler::{BuildConfig, BuildContext, Compilation, Context};
+use crate::core::compiler::{BuildConfig, BuildContext, BuildRunner, Compilation};
 use crate::core::compiler::{CompileKind, CompileMode, CompileTarget, RustcTargetData, Unit};
 use crate::core::compiler::{DefaultExecutor, Executor, UnitInterner};
 use crate::core::profiles::Profiles;
@@ -53,9 +53,9 @@ use crate::core::{PackageId, PackageSet, SourceId, TargetKind, Workspace};
 use crate::drop_println;
 use crate::ops;
 use crate::ops::resolve::WorkspaceResolve;
-use crate::util::config::Config;
+use crate::util::context::GlobalContext;
 use crate::util::interning::InternedString;
-use crate::util::{profile, CargoResult, StableHasher};
+use crate::util::{CargoResult, StableHasher};
 
 mod compile_filter;
 pub use compile_filter::{CompileFilter, FilterRule, LibRule};
@@ -98,15 +98,15 @@ pub struct CompileOptions {
     pub rustdoc_document_private_items: bool,
     /// Whether the build process should check the minimum Rust version
     /// defined in the cargo metadata for a crate.
-    pub honor_rust_version: bool,
+    pub honor_rust_version: Option<bool>,
 }
 
 impl CompileOptions {
-    pub fn new(config: &Config, mode: CompileMode) -> CargoResult<CompileOptions> {
+    pub fn new(gctx: &GlobalContext, mode: CompileMode) -> CargoResult<CompileOptions> {
         let jobs = None;
         let keep_going = false;
         Ok(CompileOptions {
-            build_config: BuildConfig::new(config, jobs, keep_going, &[], mode)?,
+            build_config: BuildConfig::new(gctx, jobs, keep_going, &[], mode)?,
             cli_features: CliFeatures::new_all(false),
             spec: ops::Packages::Packages(Vec::new()),
             filter: CompileFilter::Default {
@@ -116,7 +116,7 @@ impl CompileOptions {
             target_rustc_args: None,
             target_rustc_crate_types: None,
             rustdoc_document_private_items: false,
-            honor_rust_version: true,
+            honor_rust_version: None,
         })
     }
 }
@@ -143,6 +143,7 @@ pub fn compile_with_exec<'a>(
 }
 
 /// Like [`compile_with_exec`] but without warnings from manifest parsing.
+#[tracing::instrument(skip_all)]
 pub fn compile_ws<'a>(
     ws: &Workspace<'a>,
     options: &CompileOptions,
@@ -151,13 +152,12 @@ pub fn compile_ws<'a>(
     let interner = UnitInterner::new();
     let bcx = create_bcx(ws, options, &interner)?;
     if options.build_config.unit_graph {
-        unit_graph::emit_serialized_unit_graph(&bcx.roots, &bcx.unit_graph, ws.config())?;
+        unit_graph::emit_serialized_unit_graph(&bcx.roots, &bcx.unit_graph, ws.gctx())?;
         return Compilation::new(&bcx);
     }
-    crate::core::gc::auto_gc(bcx.config);
-    let _p = profile::start("compiling");
-    let cx = Context::new(&bcx)?;
-    cx.compile(exec)
+    crate::core::gc::auto_gc(bcx.gctx);
+    let build_runner = BuildRunner::new(&bcx)?;
+    build_runner.compile(exec)
 }
 
 /// Executes `rustc --print <VALUE>`.
@@ -173,13 +173,13 @@ pub fn print<'a>(
         ref target_rustc_args,
         ..
     } = *options;
-    let config = ws.config();
-    let rustc = config.load_global_rustc(Some(ws))?;
+    let gctx = ws.gctx();
+    let rustc = gctx.load_global_rustc(Some(ws))?;
     for (index, kind) in build_config.requested_kinds.iter().enumerate() {
         if index != 0 {
-            drop_println!(config);
+            drop_println!(gctx);
         }
-        let target_info = TargetInfo::new(config, &build_config.requested_kinds, &rustc, *kind)?;
+        let target_info = TargetInfo::new(gctx, &build_config.requested_kinds, &rustc, *kind)?;
         let mut process = rustc.process();
         process.args(&target_info.rustflags);
         if let Some(args) = target_rustc_args {
@@ -198,11 +198,12 @@ pub fn print<'a>(
 ///
 /// For how it works and what data it collects,
 /// please see the [module-level documentation](self).
-pub fn create_bcx<'a, 'cfg>(
-    ws: &'a Workspace<'cfg>,
+#[tracing::instrument(skip_all)]
+pub fn create_bcx<'a, 'gctx>(
+    ws: &'a Workspace<'gctx>,
     options: &'a CompileOptions,
     interner: &'a UnitInterner,
-) -> CargoResult<BuildContext<'a, 'cfg>> {
+) -> CargoResult<BuildContext<'a, 'gctx>> {
     let CompileOptions {
         ref build_config,
         ref spec,
@@ -214,7 +215,7 @@ pub fn create_bcx<'a, 'cfg>(
         rustdoc_document_private_items,
         honor_rust_version,
     } = *options;
-    let config = ws.config();
+    let gctx = ws.gctx();
 
     // Perform some pre-flight validation.
     match build_config.mode {
@@ -223,21 +224,21 @@ pub fn create_bcx<'a, 'cfg>(
         | CompileMode::Check { .. }
         | CompileMode::Bench
         | CompileMode::RunCustomBuild => {
-            if ws.config().get_env("RUST_FLAGS").is_ok() {
-                config.shell().warn(
+            if ws.gctx().get_env("RUST_FLAGS").is_ok() {
+                gctx.shell().warn(
                     "Cargo does not read `RUST_FLAGS` environment variable. Did you mean `RUSTFLAGS`?",
                 )?;
             }
         }
         CompileMode::Doc { .. } | CompileMode::Doctest | CompileMode::Docscrape => {
-            if ws.config().get_env("RUSTDOC_FLAGS").is_ok() {
-                config.shell().warn(
+            if ws.gctx().get_env("RUSTDOC_FLAGS").is_ok() {
+                gctx.shell().warn(
                     "Cargo does not read `RUSTDOC_FLAGS` environment variable. Did you mean `RUSTDOCFLAGS`?"
                 )?;
             }
         }
     }
-    config.validate_term_config()?;
+    gctx.validate_term_config()?;
 
     let mut target_data = RustcTargetData::new(ws, &build_config.requested_kinds)?;
 
@@ -263,7 +264,6 @@ pub fn create_bcx<'a, 'cfg>(
             HasDevUnits::No
         }
     };
-    let max_rust_version = ws.rust_version();
     let resolve = ops::resolve_ws_with_opts(
         ws,
         &mut target_data,
@@ -272,7 +272,6 @@ pub fn create_bcx<'a, 'cfg>(
         &specs,
         has_dev_units,
         crate::core::resolver::features::ForceAllTargets::No,
-        max_rust_version,
     )?;
     let WorkspaceResolve {
         mut pkg_set,
@@ -281,7 +280,7 @@ pub fn create_bcx<'a, 'cfg>(
         resolved_features,
     } = resolve;
 
-    let std_resolve_features = if let Some(crates) = &config.cli_unstable().build_std {
+    let std_resolve_features = if let Some(crates) = &gctx.cli_unstable().build_std {
         let (std_package_set, std_resolve, std_features) =
             standard_lib::resolve_std(ws, &mut target_data, &build_config, crates)?;
         pkg_set.add_set(std_package_set);
@@ -305,7 +304,7 @@ pub fn create_bcx<'a, 'cfg>(
     to_builds.sort_by_key(|p| p.package_id());
 
     for pkg in to_builds.iter() {
-        pkg.manifest().print_teapot(config);
+        pkg.manifest().print_teapot(gctx);
 
         if build_config.mode.is_any_test()
             && !ws.is_member(pkg)
@@ -335,7 +334,7 @@ pub fn create_bcx<'a, 'cfg>(
     let profiles = Profiles::new(ws, build_config.requested_profile)?;
     profiles.validate_packages(
         ws.profiles(),
-        &mut config.shell(),
+        &mut gctx.shell(),
         workspace_resolve.as_ref().unwrap_or(&resolve),
     )?;
 
@@ -378,7 +377,7 @@ pub fn create_bcx<'a, 'cfg>(
         override_rustc_crate_types(&mut units, args, interner)?;
     }
 
-    let should_scrape = build_config.mode.is_doc() && config.cli_unstable().rustdoc_scrape_examples;
+    let should_scrape = build_config.mode.is_doc() && gctx.cli_unstable().rustdoc_scrape_examples;
     let mut scrape_units = if should_scrape {
         UnitGenerator {
             mode: CompileMode::Docscrape,
@@ -389,7 +388,7 @@ pub fn create_bcx<'a, 'cfg>(
         Vec::new()
     };
 
-    let std_roots = if let Some(crates) = standard_lib::std_crates(config, Some(&units)) {
+    let std_roots = if let Some(crates) = standard_lib::std_crates(gctx, Some(&units)) {
         let (std_resolve, std_features) = std_resolve_features.as_ref().unwrap();
         standard_lib::generate_std_roots(
             &crates,
@@ -429,24 +428,17 @@ pub fn create_bcx<'a, 'cfg>(
         .requested_kinds
         .iter()
         .any(CompileKind::is_host);
-    let should_share_deps = host_kind_requested
-        || config.cli_unstable().bindeps
-            && unit_graph
-                .iter()
-                .any(|(unit, _)| unit.artifact_target_for_features.is_some());
-    if should_share_deps {
-        // Rebuild the unit graph, replacing the explicit host targets with
-        // CompileKind::Host, removing `artifact_target_for_features` and merging any dependencies
-        // shared with build and artifact dependencies.
-        (units, scrape_units, unit_graph) = rebuild_unit_graph_shared(
-            interner,
-            unit_graph,
-            &units,
-            &scrape_units,
-            host_kind_requested.then_some(explicit_host_kind),
-            &target_data,
-        );
-    }
+    // Rebuild the unit graph, replacing the explicit host targets with
+    // CompileKind::Host, removing `artifact_target_for_features` and merging any dependencies
+    // shared with build and artifact dependencies.
+    (units, scrape_units, unit_graph) = rebuild_unit_graph_shared(
+        interner,
+        unit_graph,
+        &units,
+        &scrape_units,
+        host_kind_requested.then_some(explicit_host_kind),
+        &target_data,
+    );
 
     let mut extra_compiler_args = HashMap::new();
     if let Some(args) = extra_args {
@@ -482,54 +474,54 @@ pub fn create_bcx<'a, 'cfg>(
             .extend(args);
     }
 
-    if honor_rust_version {
-        // Remove any pre-release identifiers for easier comparison
-        let current_version = &target_data.rustc.version;
-        let untagged_version = semver::Version::new(
-            current_version.major,
-            current_version.minor,
-            current_version.patch,
-        );
+    if honor_rust_version.unwrap_or(true) {
+        let rustc_version = target_data.rustc.version.clone().into();
 
+        let mut incompatible = Vec::new();
+        let mut local_incompatible = false;
         for unit in unit_graph.keys() {
-            let Some(version) = unit.pkg.rust_version() else {
+            let Some(pkg_msrv) = unit.pkg.rust_version() else {
                 continue;
             };
 
-            let req = version.to_caret_req();
-            if req.matches(&untagged_version) {
+            if pkg_msrv.is_compatible_with(&rustc_version) {
                 continue;
             }
 
-            let guidance = if ws.is_ephemeral() {
-                if ws.ignore_lock() {
-                    "Try re-running cargo install with `--locked`".to_string()
-                } else {
-                    String::new()
-                }
-            } else if !unit.is_local() {
-                format!(
-                    "Either upgrade to rustc {} or newer, or use\n\
-                     cargo update {}@{} --precise ver\n\
-                     where `ver` is the latest version of `{}` supporting rustc {}",
-                    version,
-                    unit.pkg.name(),
-                    unit.pkg.version(),
-                    unit.pkg.name(),
-                    current_version,
-                )
-            } else {
-                String::new()
-            };
+            local_incompatible |= unit.is_local();
+            incompatible.push((unit, pkg_msrv));
+        }
+        if !incompatible.is_empty() {
+            use std::fmt::Write as _;
 
-            anyhow::bail!(
-                "package `{}` cannot be built because it requires rustc {} or newer, \
-                 while the currently active rustc version is {}\n{}",
-                unit.pkg,
-                version,
-                current_version,
-                guidance,
+            let plural = if incompatible.len() == 1 { "" } else { "s" };
+            let mut message = format!(
+                "rustc {rustc_version} is not supported by the following package{plural}:\n"
             );
+            incompatible.sort_by_key(|(unit, _)| (unit.pkg.name(), unit.pkg.version()));
+            for (unit, msrv) in incompatible {
+                let name = &unit.pkg.name();
+                let version = &unit.pkg.version();
+                writeln!(&mut message, "  {name}@{version} requires rustc {msrv}").unwrap();
+            }
+            if ws.is_ephemeral() {
+                if ws.ignore_lock() {
+                    writeln!(
+                        &mut message,
+                        "Try re-running `cargo install` with `--locked`"
+                    )
+                    .unwrap();
+                }
+            } else if !local_incompatible {
+                writeln!(
+                    &mut message,
+                    "Either upgrade rustc or select compatible dependency versions with
+`cargo update <name>@<current-ver> --precise <compatible-ver>`
+where `<compatible-ver>` is the latest version supporting rustc {rustc_version}",
+                )
+                .unwrap();
+            }
+            return Err(anyhow::Error::msg(message));
         }
     }
 
@@ -548,7 +540,8 @@ pub fn create_bcx<'a, 'cfg>(
     Ok(bcx)
 }
 
-/// This is used to rebuild the unit graph, sharing host dependencies if possible.
+/// This is used to rebuild the unit graph, sharing host dependencies if possible,
+/// and applying other unit adjustments based on the whole graph.
 ///
 /// This will translate any unit's `CompileKind::Target(host)` to
 /// `CompileKind::Host` if `to_host` is not `None` and the kind is equal to `to_host`.
@@ -570,6 +563,14 @@ pub fn create_bcx<'a, 'cfg>(
 /// to the `Unit`, this allows the `CompileKind` to be changed back to `Host`
 /// and `artifact_target_for_features` to be removed without fear of an unwanted
 /// collision for build or artifact dependencies.
+///
+/// This is also responsible for adjusting the `strip` profile option to
+/// opportunistically strip if debug is 0 for all dependencies. This helps
+/// remove debuginfo added by the standard library.
+///
+/// This is also responsible for adjusting the `debug` setting for host
+/// dependencies, turning off debug if the user has not explicitly enabled it,
+/// and the unit is not shared with a target unit.
 fn rebuild_unit_graph_shared(
     interner: &UnitInterner,
     unit_graph: UnitGraph,
