@@ -12,6 +12,7 @@ fn feature_required() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
                 im-a-teapot = true
             "#,
@@ -77,6 +78,7 @@ fn feature_required_dependency() {
                 [package]
                 name = "bar"
                 version = "0.1.0"
+                edition = "2015"
                 im-a-teapot = true
             "#,
         )
@@ -89,6 +91,7 @@ fn feature_required_dependency() {
                 [package]
                 name = "foo"
                 version = "0.1.0"
+                edition = "2015"
 
                 [dependencies]
                 bar = "1.0"
@@ -103,6 +106,7 @@ fn feature_required_dependency() {
         .with_stderr(
             "\
 [UPDATING] [..]
+[LOCKING] 2 packages to latest compatible versions
 [DOWNLOADING] [..]
 [DOWNLOADED] bar v1.0.0 [..]
 error: failed to download replaced source registry `crates-io`
@@ -168,6 +172,7 @@ fn unknown_feature() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
             "#,
         )
@@ -197,6 +202,7 @@ fn stable_feature_warns() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
             "#,
         )
@@ -226,6 +232,7 @@ fn allow_features() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
                 im-a-teapot = true
             "#,
@@ -289,6 +296,7 @@ fn allow_features_to_rustc() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
             "#,
         )
@@ -329,6 +337,7 @@ fn allow_features_in_cfg() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
                 im-a-teapot = true
             "#,
@@ -377,7 +386,7 @@ error: the feature `unstable-options` is not in the list of allowed features: [p
         )
         .run();
 
-    // -Zallow-features overrides .cargo/config
+    // -Zallow-features overrides .cargo/config.toml
     p.cargo("-Zallow-features=test-dummy-unstable -Zprint-im-a-teapot check")
         .masquerade_as_nightly_cargo(&[
             "allow-features",
@@ -421,6 +430,7 @@ fn nightly_feature_requires_nightly() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
                 im-a-teapot = true
             "#,
@@ -463,6 +473,7 @@ fn nightly_feature_requires_nightly_in_dep() {
                 [package]
                 name = "b"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -478,6 +489,7 @@ fn nightly_feature_requires_nightly_in_dep() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
                 im-a-teapot = true
             "#,
@@ -488,6 +500,7 @@ fn nightly_feature_requires_nightly_in_dep() {
         .masquerade_as_nightly_cargo(&["test-dummy-unstable"])
         .with_stderr(
             "\
+[LOCKING] 2 packages to latest compatible versions
 [CHECKING] a [..]
 [CHECKING] b [..]
 [FINISHED] [..]
@@ -532,6 +545,7 @@ fn cant_publish() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
                 im-a-teapot = true
             "#,
@@ -576,6 +590,7 @@ fn z_flags_rejected() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
                 im-a-teapot = true
             "#,
@@ -631,6 +646,7 @@ fn publish_allowed() {
                 [package]
                 name = "a"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
             "#,
         )
@@ -652,7 +668,7 @@ fn publish_allowed() {
 [PACKAGED] [..]
 [UPLOADING] a v0.0.1 [..]
 [UPLOADED] a v0.0.1 to registry `crates-io`
-note: Waiting for `a v0.0.1` to be available at registry `crates-io`.
+[NOTE] waiting for `a v0.0.1` to be available at registry `crates-io`.
 You may press ctrl-c to skip waiting; the crate should be available shortly.
 [PUBLISHED] a v0.0.1 at registry `crates-io`
 ",
@@ -669,6 +685,7 @@ fn wrong_position() {
                 [package]
                 name = "foo"
                 version = "0.1.0"
+                edition = "2015"
                 cargo-features = ["test-dummy-unstable"]
             "#,
         )
@@ -680,9 +697,9 @@ fn wrong_position() {
         .with_stderr(
             "\
 [ERROR] the field `cargo-features` should be set at the top of Cargo.toml before any tables
- --> Cargo.toml:5:34
+ --> Cargo.toml:6:34
   |
-5 |                 cargo-features = [\"test-dummy-unstable\"]
+6 |                 cargo-features = [\"test-dummy-unstable\"]
   |                                  ^^^^^^^^^^^^^^^^^^^^^^^
   |
 ",
