@@ -1,10 +1,12 @@
-use cargo_test_support::curr_dir;
+use cargo_test_support::current_dir;
+use cargo_test_support::file;
+use cargo_test_support::str;
 use cargo_test_support::CargoCommand;
 use cargo_test_support::Project;
 
 #[cargo_test]
 fn case() {
-    let project = Project::from_template(curr_dir!().join("in"));
+    let project = Project::from_template(current_dir!().join("in"));
     let project_root = project.root();
     let cwd = &project_root;
 
@@ -14,6 +16,6 @@ fn case() {
         .current_dir(cwd)
         .assert()
         .code(1)
-        .stdout_matches_path(curr_dir!().join("stdout.log"))
-        .stderr_matches_path(curr_dir!().join("stderr.log"));
+        .stdout_eq(str![""])
+        .stderr_eq(file!["stderr.term.svg"]);
 }
