@@ -16,6 +16,7 @@ fn build_bin_default_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -71,6 +72,7 @@ fn build_bin_arg_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -97,6 +99,7 @@ fn build_bin_multiple_required_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -142,6 +145,7 @@ fn build_example_default_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -179,6 +183,7 @@ fn build_example_arg_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -205,6 +210,7 @@ fn build_example_multiple_required_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -275,6 +281,7 @@ fn test_default_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -293,21 +300,21 @@ fn test_default_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] [..] (target/debug/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("test test ... ok")
         .run();
 
     p.cargo("test --no-default-features")
-        .with_stderr("[FINISHED] test [unoptimized + debuginfo] target(s) in [..]")
+        .with_stderr("[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]")
         .with_stdout("")
         .run();
 
     p.cargo("test --test=foo")
         .with_stderr(
             "\
-[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] [..] (target/debug/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("test test ... ok")
@@ -333,6 +340,7 @@ fn test_arg_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -350,7 +358,7 @@ fn test_arg_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] [..] (target/debug/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("test test ... ok")
@@ -366,6 +374,7 @@ fn test_multiple_required_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -391,7 +400,7 @@ fn test_multiple_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] [..] (target/debug/deps/foo_2-[..][EXE])",
         )
         .with_stdout_contains("test test ... ok")
@@ -401,7 +410,7 @@ fn test_multiple_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] [..] (target/debug/deps/foo_1-[..][EXE])
 [RUNNING] [..] (target/debug/deps/foo_2-[..][EXE])",
         )
@@ -409,7 +418,7 @@ fn test_multiple_required_features() {
         .run();
 
     p.cargo("test --no-default-features")
-        .with_stderr("[FINISHED] test [unoptimized + debuginfo] target(s) in [..]")
+        .with_stderr("[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]")
         .with_stdout("")
         .run();
 }
@@ -423,6 +432,7 @@ fn bench_default_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -451,21 +461,21 @@ fn bench_default_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] bench [optimized] target(s) in [..]
+[FINISHED] `bench` profile [optimized] target(s) in [..]
 [RUNNING] [..] (target/release/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("test bench ... bench: [..]")
         .run();
 
     p.cargo("bench --no-default-features")
-        .with_stderr("[FINISHED] bench [optimized] target(s) in [..]".to_string())
+        .with_stderr("[FINISHED] `bench` profile [optimized] target(s) in [..]".to_string())
         .with_stdout("")
         .run();
 
     p.cargo("bench --bench=foo")
         .with_stderr(
             "\
-[FINISHED] bench [optimized] target(s) in [..]
+[FINISHED] `bench` profile [optimized] target(s) in [..]
 [RUNNING] [..] (target/release/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("test bench ... bench: [..]")
@@ -491,6 +501,7 @@ fn bench_arg_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -518,7 +529,7 @@ fn bench_arg_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] bench [optimized] target(s) in [..]
+[FINISHED] `bench` profile [optimized] target(s) in [..]
 [RUNNING] [..] (target/release/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("test bench ... bench: [..]")
@@ -534,6 +545,7 @@ fn bench_multiple_required_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -579,7 +591,7 @@ fn bench_multiple_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] bench [optimized] target(s) in [..]
+[FINISHED] `bench` profile [optimized] target(s) in [..]
 [RUNNING] [..] (target/release/deps/foo_2-[..][EXE])",
         )
         .with_stdout_contains("test bench ... bench: [..]")
@@ -589,7 +601,7 @@ fn bench_multiple_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] bench [optimized] target(s) in [..]
+[FINISHED] `bench` profile [optimized] target(s) in [..]
 [RUNNING] [..] (target/release/deps/foo_1-[..][EXE])
 [RUNNING] [..] (target/release/deps/foo_2-[..][EXE])",
         )
@@ -597,7 +609,7 @@ fn bench_multiple_required_features() {
         .run();
 
     p.cargo("bench --no-default-features")
-        .with_stderr("[FINISHED] bench [optimized] target(s) in [..]")
+        .with_stderr("[FINISHED] `bench` profile [optimized] target(s) in [..]")
         .with_stdout("")
         .run();
 }
@@ -611,6 +623,7 @@ fn install_default_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -638,7 +651,7 @@ fn install_default_features() {
         .with_stderr(
             "\
 [INSTALLING] foo v0.0.1 ([..])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] `release` profile [optimized] target(s) in [..]
 [WARNING] none of the package's binaries are available for install using the selected features
   bin \"foo\" requires the features: `a`
   example \"foo\" requires the features: `a`
@@ -701,6 +714,7 @@ fn install_arg_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -728,6 +742,7 @@ fn install_multiple_required_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -796,7 +811,7 @@ fn install_multiple_required_features() {
         .with_stderr(
             "\
 [INSTALLING] foo v0.0.1 ([..])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] `release` profile [optimized] target(s) in [..]
 [WARNING] none of the package's binaries are available for install using the selected features
   bin \"foo_1\" requires the features: `b`, `c`
   bin \"foo_2\" requires the features: `a`
@@ -810,8 +825,8 @@ Consider enabling some of the needed features by passing, e.g., `--features=\"b
         .with_stderr(
             "\
 [INSTALLING] foo v0.0.1 ([..])
-[WARNING] Target filter `bins` specified, but no targets matched. This is a no-op
-[FINISHED] release [optimized] target(s) in [..]
+[WARNING] target filter `bins` specified, but no targets matched; this is a no-op
+[FINISHED] `release` profile [optimized] target(s) in [..]
 [WARNING] none of the package's binaries are available for install using the selected features
   bin \"foo_1\" requires the features: `b`, `c`
   bin \"foo_2\" requires the features: `a`
@@ -825,8 +840,8 @@ Consider enabling some of the needed features by passing, e.g., `--features=\"b
         .with_stderr(
             "\
 [INSTALLING] foo v0.0.1 ([..])
-[WARNING] Target filter `examples` specified, but no targets matched. This is a no-op
-[FINISHED] release [optimized] target(s) in [..]
+[WARNING] target filter `examples` specified, but no targets matched; this is a no-op
+[FINISHED] `release` profile [optimized] target(s) in [..]
 [WARNING] none of the package's binaries are available for install using the selected features
   bin \"foo_1\" requires the features: `b`, `c`
   bin \"foo_2\" requires the features: `a`
@@ -840,8 +855,8 @@ Consider enabling some of the needed features by passing, e.g., `--features=\"b
         .with_stderr(
             "\
 [INSTALLING] foo v0.0.1 ([..])
-[WARNING] Target filters `bins`, `examples` specified, but no targets matched. This is a no-op
-[FINISHED] release [optimized] target(s) in [..]
+[WARNING] target filters `bins`, `examples` specified, but no targets matched; this is a no-op
+[FINISHED] `release` profile [optimized] target(s) in [..]
 [WARNING] none of the package's binaries are available for install using the selected features
   bin \"foo_1\" requires the features: `b`, `c`
   bin \"foo_2\" requires the features: `a`
@@ -866,6 +881,7 @@ fn dep_feature_in_toml() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -908,6 +924,7 @@ fn dep_feature_in_toml() {
                 [package]
                 name = "bar"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -932,7 +949,7 @@ fn dep_feature_in_toml() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] [..] (target/debug/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("test test ... ok")
@@ -945,7 +962,7 @@ fn dep_feature_in_toml() {
                 "\
 [COMPILING] bar v0.0.1 ([CWD]/bar)
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] bench [optimized] target(s) in [..]
+[FINISHED] `bench` profile [optimized] target(s) in [..]
 [RUNNING] [..] (target/release/deps/foo-[..][EXE])",
             )
             .with_stdout_contains("test bench ... bench: [..]")
@@ -967,6 +984,7 @@ fn dep_feature_in_cmd_line() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [dependencies]
@@ -1019,6 +1037,7 @@ fn dep_feature_in_cmd_line() {
                 [package]
                 name = "bar"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -1029,7 +1048,13 @@ fn dep_feature_in_cmd_line() {
         .build();
 
     // This is a no-op
-    p.cargo("build").with_stderr("[FINISHED] dev [..]").run();
+    p.cargo("build")
+        .with_stderr(
+            "\
+[LOCKING] 2 packages to latest compatible versions
+[FINISHED] `dev` profile [..]",
+        )
+        .run();
     assert!(!p.bin("foo").is_file());
 
     // bin
@@ -1063,7 +1088,7 @@ Consider enabling them by passing, e.g., `--features=\"bar/a\"`
     // test
     // This is a no-op, since no tests are enabled
     p.cargo("test")
-        .with_stderr("[FINISHED] test [unoptimized + debuginfo] target(s) in [..]")
+        .with_stderr("[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]")
         .with_stdout("")
         .run();
 
@@ -1074,7 +1099,7 @@ Consider enabling them by passing, e.g., `--features=\"bar/a\"`
             "\
 [COMPILING] bar v0.0.1 ([CWD]/bar)
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] [..] (target/debug/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("test bin_is_built ... ok")
@@ -1083,7 +1108,7 @@ Consider enabling them by passing, e.g., `--features=\"bar/a\"`
     // bench
     if is_nightly() {
         p.cargo("bench")
-            .with_stderr("[FINISHED] bench [optimized] target(s) in [..]")
+            .with_stderr("[FINISHED] `bench` profile [optimized] target(s) in [..]")
             .with_stdout("")
             .run();
 
@@ -1092,7 +1117,7 @@ Consider enabling them by passing, e.g., `--features=\"bar/a\"`
                 "\
 [COMPILING] bar v0.0.1 ([CWD]/bar)
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] bench [optimized] target(s) in [..]
+[FINISHED] `bench` profile [optimized] target(s) in [..]
 [RUNNING] [..] (target/release/deps/foo-[..][EXE])",
             )
             .with_stdout_contains("test bench ... bench: [..]")
@@ -1104,7 +1129,8 @@ Consider enabling them by passing, e.g., `--features=\"bar/a\"`
         .with_stderr(
             "\
 [INSTALLING] foo v0.0.1 ([..])
-[FINISHED] release [optimized] target(s) in [..]
+[LOCKING] 2 packages to latest compatible versions
+[FINISHED] `release` profile [optimized] target(s) in [..]
 [WARNING] none of the package's binaries are available for install using the selected features
   bin \"foo\" requires the features: `bar/a`
   example \"foo\" requires the features: `bar/a`
@@ -1128,6 +1154,7 @@ fn test_skips_compiling_bin_with_missing_required_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -1148,7 +1175,7 @@ fn test_skips_compiling_bin_with_missing_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] `test` profile [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] [..] (target/debug/deps/foo-[..][EXE])",
         )
         .with_stdout_contains("running 0 tests")
@@ -1168,7 +1195,7 @@ error[E0463]: can't find crate for `bar`",
             .with_stderr(
                 "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] bench [optimized] target(s) in [..]
+[FINISHED] `bench` profile [optimized] target(s) in [..]
 [RUNNING] [..] (target/release/deps/foo-[..][EXE])",
             )
             .with_stdout_contains("running 0 tests")
@@ -1194,6 +1221,7 @@ fn run_default() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -1231,6 +1259,7 @@ fn run_default_multiple_required_features() {
                 [package]
                 name = "foo"
                 version = "0.0.1"
+                edition = "2015"
                 authors = []
 
                 [features]
@@ -1348,6 +1377,7 @@ fn renamed_required_features() {
         .with_status(101)
         .with_stderr(
             "\
+[LOCKING] 3 packages to latest compatible versions
 [ERROR] target `x` in package `foo` requires the features: `a1/f1`
 Consider enabling them by passing, e.g., `--features=\"a1/f1\"`
 ",
@@ -1441,15 +1471,15 @@ fn truncated_install_warning_message() {
 
     p.cargo("install --path .").with_stderr("\
 [INSTALLING] foo v0.1.0 ([..])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] `release` profile [optimized] target(s) in [..]
 [WARNING] none of the package's binaries are available for install using the selected features
   bin \"foo1\" requires the features: `feature1`, `feature2`, `feature3`
+  bin \"foo10\" requires the features: `feature1`, `feature2`, `feature3`, `feature4`, `feature5`
   bin \"foo2\" requires the features: `feature2`
   bin \"foo3\" requires the features: `feature3`
   bin \"foo4\" requires the features: `feature4`, `feature1`
   bin \"foo5\" requires the features: `feature1`, `feature2`, `feature3`, `feature4`, `feature5`
   bin \"foo6\" requires the features: `feature1`, `feature2`, `feature3`, `feature4`, `feature5`
-  bin \"foo7\" requires the features: `feature1`, `feature2`, `feature3`, `feature4`, `feature5`
 4 more targets also requires features not enabled. See them in the Cargo.toml file.
 Consider enabling some of the needed features by passing, e.g., `--features=\"feature1 feature2 feature3\"`").run();
 }
