@@ -1,9 +1,9 @@
-// needs-llvm-components: x86
-// compile-flags: --target=i686-pc-windows-msvc --crate-type=rlib -Cno-prepopulate-passes
+//@ needs-llvm-components: x86
+//@ compile-flags: --target=i686-pc-windows-msvc --crate-type=rlib -Cno-prepopulate-passes
 #![no_core]
 #![feature(no_core, lang_items, c_unwind)]
-#[lang="sized"]
-trait Sized { }
+#[lang = "sized"]
+trait Sized {}
 
 // Test that `nounwind` attributes are correctly applied to exported `fastcall` and
 // `fastcall-unwind` extern functions. `fastcall-unwind` functions MUST NOT have this attribute. We
@@ -11,13 +11,11 @@ trait Sized { }
 
 // CHECK: @rust_item_that_cannot_unwind() unnamed_addr #0 {
 #[no_mangle]
-pub extern "fastcall" fn rust_item_that_cannot_unwind() {
-}
+pub extern "fastcall" fn rust_item_that_cannot_unwind() {}
 
 // CHECK: @rust_item_that_can_unwind() unnamed_addr #1 {
 #[no_mangle]
-pub extern "fastcall-unwind" fn rust_item_that_can_unwind() {
-}
+pub extern "fastcall-unwind" fn rust_item_that_can_unwind() {}
 
 // Now, make some assertions that the LLVM attributes for these functions are correct.  First, make
 // sure that the first item is correctly marked with the `nounwind` attribute:
