@@ -1,6 +1,5 @@
-// run-pass
-// compile-flags: -g
-// ignore-asmjs wasm2js does not support source maps yet
+//@ run-pass
+//@ compile-flags: -g
 
 #![feature(coroutines, coroutine_trait)]
 
@@ -14,7 +13,7 @@ impl Database {
     }
 
     fn check_connection(&self) -> impl Coroutine<Yield = (), Return = ()> + '_ {
-        move || {
+        #[coroutine] move || {
             let iter = self.get_connection();
             for i in iter {
                 yield i
