@@ -1,6 +1,3 @@
-#![feature(exclusive_range_pattern)]
-#![allow(illegal_floating_point_literal_pattern)]
-
 macro_rules! m {
     ($s:expr, $($t:tt)+) => {
         match $s { $($t)+ => {} }
