@@ -1,14 +1,12 @@
-// check-fail
-//[next] compile-flags: -Ztrait-solver=next
+//@ check-fail
 
 #![feature(transmutability)]
 mod assert {
     use std::mem::{Assume, BikeshedIntrinsicFrom};
-    pub struct Context;
 
     pub fn is_transmutable<Src, Dst>()
     where
-        Dst: BikeshedIntrinsicFrom<Src, Context, { Assume::SAFETY }>
+        Dst: BikeshedIntrinsicFrom<Src, { Assume::SAFETY }>
     {}
 }
 
